/*
 * Some utilities for cloudnet applications.
 *
 * HISTORY
 * 2003-04-12 ao started.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNAPP_H)
#define CNAPP_H


/************************************************************************/
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>


/************************************************************************/
/************************************************************************/


/* These must be defined in another file. */

extern char * PROGNAME;

extern void printSynopsis(FILE * outfile, int exitProg);


/************************************************************************/
/************************************************************************/


/* Options for 'printError()'. */
enum EPrintErrorOptions
{
  PE_EXIT           = 1,
  PE_PRINT_SYNOPSIS = 2
};


/************************************************************************/
/************************************************************************/


/* Print an error/warning/diagnostic message on stderr.
 * PRE: 'option' is an ORed combination of 'EPrintErrorOptions'.
 * 'fmt' is a 'printf' type format string followed by arguments.
 * POST: prints a line to stderr beginning with the program name
 * followed by a colon then 'fmt' then optionally by a system
 * error message.
 * NOTES: do not terminate 'fmt' with a '\n'.
 */
void printError(int option, char * fmt, ...);


/* Get option argument from command line.
 *
 * PRE: 'option' is the option name to use in printing error messages.
 * 'argc' and 'argv' and the arguments to 'main()'.
 * 'argn' is a pointer to the argument index in 'argv'.
 *
 * POST: on success, returns the next argument from the
 * command line and *argn is incremented.
 * On failure, an error message is printed and 'printSynopsis()' is
 * called then the program exits (the function does not return).
 */
char * getOptArg(char * option, int argc, char * argv[], int * argn);


/************************************************************************/
/************************************************************************/


#endif
