/*
 * Stuff for the intermediate dataset format.
 *
 * HISTORY
 * 2004-03-15 ao
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNIM_H)
#define CNIM_H


/************************************************************************/
/************************************************************************/


#include <netcdf.h>
#include "cnnetcdf.h"


/************************************************************************/
/************************************************************************/


/* Structure to hold the entire contents of an IM file. */
typedef struct
{
  /* Global attributes. */
  struct
  {
    int   site;
    int   institute;
    int   model;
    int   fileType;
    int   initTime;
    int   utcOffset;
  } ga;

  /* Dimensions. */
  struct
  {
    int   time;
    int   level;
  } dims;

  /* Variables. */
  struct
  {
    float latitude;
    float longitude;

    /* 'dims.time' is the number of elements.
     * Time is minutes from 'ga.initTime'.
     * The index into this corresponds to the index into the first
     * dimension of 'vars.vars[].data'.
     */
    int   time[CN_NC_MAX_TIME_DIM];

    /* For each model parameter in the file. */
    struct
    {
      int   varId;  /* one of 'cnEVariableIds' from 'cnmodels.h'. */
      float data[CN_NC_MAX_TIME_DIM] [CN_NC_MAX_LEVEL_DIM];
    }     vars[CN_NC_MAX_VARS];

    /* The number of valid elements in 'vars'. */
    int   numVars;
  } vars;
} cnTIM;


/************************************************************************/
/************************************************************************/


/* Create a new 'cnTIM' object, read the file and return the object. */
void cnImRead(char * filename, cnTIM * im/*OUT*/);


/* Search 'im->vars' for 'varId'
 * POST: returns -1 if 'varId' is not found otherwise returns the index
 * into 'im->vars'.
 */
int cnImSearchVars(cnTIM * im, int varId);


/************************************************************************/
/************************************************************************/


#endif
