/*
 * Constants for various model related stuff.
 *
 * HISTORY
 * 2003-04-03 ao
 * 2003-08-06 ao added CN_VAR_OMEGA.
 * 2003-10-14 ao added CN_RACMO in cnEModelSources.
 * 2004-01-05 ao added CN_VAR_SPEC_DRY_GAS_ATTEN.
 * 2004-02-25 ao major revision.
 * 2005-05-11 eoc added DWD model.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNMODELS_H)
#define CNMODELS_H


/************************************************************************/
/************************************************************************/


/* Institutes whose models are being used. */
enum cnEModelInstitutes
{
  CN_MI_FIRST                = 0,

  CN_MI_MO                   = CN_MI_FIRST,
  CN_MI_EC,
  CN_MI_MF,
  CN_MI_KN,
  CN_MI_SM,
  CN_MI_DW,

  CN_MI_COUNT
};


enum cnEModels
{
  CN_ML_FIRST                = 0,

  CN_ML_MOUMM                = CN_ML_FIRST,
  CN_ML_MOUMG,
  CN_ML_ECIFS,
  CN_ML_MFARP,
  CN_ML_KNHIR,
  CN_ML_KNRAC,
  CN_ML_SMRCA,
  CN_ML_DWDLM,
  CN_ML_MOUNA,
  CN_ML_MOUHR,

  CN_ML_COUNT
};


/* File types. */
enum cnEFileTypes
{
  CN_FT_FIRST                = 0,

  CN_FT_VAR                  = CN_FT_FIRST,
  CN_FT_FLX,
  CN_FT_SFC,

  CN_FT_COUNT
};


/* Model variables' identifiers.
 * These must be >= 0.
 */
enum cnEVariableIds
{
  CN_VAR_FIRST               = 0,
  CN_VAR_PRESSURE            = CN_VAR_FIRST,
  CN_VAR_HEIGHT,
  CN_VAR_HEIGHT_AMSL,
  CN_VAR_WIND_U,
  CN_VAR_WIND_V,
  CN_VAR_WIND_W,
  CN_VAR_OMEGA,
  CN_VAR_TKE,
  CN_VAR_TEMPERATURE,
  CN_VAR_Q,
  CN_VAR_RH,
  CN_VAR_QL,
  CN_VAR_QI,
  CN_VAR_QR,
  CN_VAR_QS,
  CN_VAR_QL_TOT,
  CN_VAR_QI_TOT,
  CN_VAR_LS_CLOUD_FRACTION,
  CN_VAR_CONV_CLOUD_FRACTION,
  CN_VAR_TOTAL_CLOUD_FRACTION,
  CN_VAR_TOTAL_CLOUD_FRACTION_XUR,
  CN_VAR_REFF,
  CN_VAR_LAST,
  CN_VAR_COUNT               = CN_VAR_LAST - CN_VAR_FIRST,

  CN_FLX_FIRST               = CN_VAR_LAST,
  CN_FLX_HEIGHT              = CN_FLX_FIRST,
  CN_FLX_HEIGHT_AMSL,
  CN_FLX_NET_SW,
  CN_FLX_NET_LW,
  CN_FLX_DOWN_SENS_HEAT,
  CN_FLX_TURB_MOIST,
  CN_FLX_LS_RAIN,
  CN_FLX_LS_SNOW,
  CN_FLX_CONV_RAIN,
  CN_FLX_CONV_SNOW,
  CN_FLX_TURB_MOM_U,
  CN_FLX_TURB_MOM_V,
  CN_FLX_LAST,
  CN_FLX_COUNT               = CN_FLX_LAST - CN_FLX_FIRST,

  CN_SFC_FIRST               = CN_FLX_LAST,
  CN_SFC_HEIGHT_AMSL         = CN_SFC_FIRST,
  CN_SFC_PRESSURE,
  CN_SFC_TEMPERATURE,
  CN_SFC_NET_SW,
  CN_SFC_NET_LW,
  CN_SFC_DOWN_SW,
  CN_SFC_DOWN_LW,
  CN_SFC_UP_SW,
  CN_SFC_UP_LW,
  CN_TOA_UP_SW,
  CN_TOA_DOWN_SW,
  CN_TOA_UP_LW,
  CN_SFC_CS_DOWN_SW,
  CN_SFC_CS_DOWN_LW,
  CN_SFC_DOWN_LAT_HEAT_FLX,
  CN_SFC_DOWN_SENS_HEAT_FLX,
  CN_SFC_LS_RAIN,
  CN_SFC_LS_RAIN_FLX,
  CN_SFC_CONV_RAIN,
  CN_SFC_CONV_RAIN_FLX,
  CN_SFC_LS_SNOW,
  CN_SFC_LS_SNOW_FLX,
  CN_SFC_CONV_SNOW,
  CN_SFC_CONV_SNOW_FLX,
  CN_SFC_LS_PRECIP_FRACTION,
  CN_SFC_LS_CLOUD_FRACTION,
  CN_SFC_CONV_CLOUD_FRACTION,
  CN_SFC_TOTAL_CLOUD_FRACTION,
  CN_SFC_BL_HEIGHT,
  CN_SFC_ALBEDO,
  CN_SFC_TEMP_2M,
  CN_SFC_Q_2M,
  CN_SFC_ROUGH_MOM,
  CN_SFC_ROUGH_HEAT,
  CN_SFC_SKIN_TEMP,
  CN_SFC_WIND_U_10M,
  CN_SFC_WIND_V_10M,
  CN_SFC_GEOPOTENTIAL,
  CN_SFC_GEOPOTENTIAL_HEIGHT,
  CN_SFC_LAST,
  CN_SFC_COUNT               = CN_SFC_LAST - CN_SFC_FIRST,

  CN_MWP_FIRST               = CN_SFC_LAST,
  CN_MWP_GAS_ATTEN           = CN_MWP_FIRST,
  CN_MWP_SPEC_GAS_ATTEN,
  CN_MWP_SPEC_SAT_GAS_ATTEN,
  CN_MWP_SPEC_DRY_GAS_ATTEN,
  CN_MWP_K2,
  CN_MWP_SPEC_LIQ_ATTEN,
  CN_MWP_LAST,
  CN_MWP_COUNT               = CN_MWP_LAST - CN_MWP_FIRST,

  /**********************************************************************/

  /* Total number of variable IDs. */
  CN_VARIABLE_ID_COUNT       = CN_MWP_LAST - CN_VAR_FIRST

};


/************************************************************************/
/************************************************************************/


#define cnMCheckRange(lo, hi, i) (( (lo) <= (i) && (i) < (hi) ) ? 1 : 0)
/* Test that 'i' is in the range lo<=i<hi.
 * 'i' is evaluated twice.
 */


#define cnMCheckModelInstitutes(modelInst) \
cnMCheckRange(CN_MI_FIRST, CN_MI_COUNT, modelInst)


#define cnMCheckModel(model) \
cnMCheckRange(CN_ML_FIRST, CN_ML_COUNT, model)


#define cnMCheckFileType(fileType) \
cnMCheckRange(CN_FT_FIRST, CN_FT_COUNT, fileType)


/************************************************************************/
/************************************************************************/


#endif
