


#include "cnnetcdf.h"


/************************************************************************/
/************************************************************************/


int ncRetValue = 0;


/************************************************************************/
/************************************************************************/


ncTVariable ncVariables[CN_VARIABLE_ID_COUNT] =
{
  { "pressure", 0, "Pa",
    "Pressure",
    "air_pressure",
    NULL },

  { "height", 3, "m",
    "Height above ground",
    "height",
    NULL },

  { "height_amsl", 3, "m",
    "Height above mean sea level",
    NULL,
    NULL },

  { "uwind", 6, "m s-1",
    "Zonal wind",
    "eastward_wind",
    NULL },

  { "vwind", 6, "m s-1",
    "Meridional wind",
    "northward_wind",
    NULL },

  { "wwind", 6, "m s-1",
    "Vertical wind",
    "upward_wind",
    NULL },

  { "omega", 6, "Pa s-1",
    "Vertical wind in pressure coordinates",
    "omega",
    NULL },

  { "tke", 6, "m2 s-2",
    "Turbulent Kinetic energy",
    "turbulent_kinetic_energy",
    NULL },

  { "temperature", 2, "K",
    "Temperature",
    "air_temperature",
    NULL },

  { "q", 8, "1",
    "Specific humidity",
    "specific_humidity",
    NULL },

  { "rh", 8, "1",
    "Relative humidity",
    "relative_humidity",
    "With respect to liquid above 0 degrees C and with respect to ice below 0 degrees C." },

  { "ql", 8, "1",
    "Gridbox-mean liquid water mixing ratio",
    "mass_fraction_of_cloud_liquid_water_in_air",
    NULL },

  { "qi", 8, "1",
    "Gridbox-mean ice water mixing ratio",
    "mass_fraction_of_cloud_ice_in_air",
    NULL },

  { "qr", 8, "1",
    "Gridbox-mean rain water mixing ratio",
    "mass_fraction_of_rain_water_in_air",
    NULL },

  { "qs", 8, "1",
    "Gridbox-mean snow mixing ratio",
    "mass_fraction_of_snow_in_air",
    NULL },

  { "ql_tot", 8, "1",
    "Gridbox-mean total liquid water mixing ratio",
    "mass_fraction_of_total_cloud_liquid_water_in_air",
    "Gridbox-mean total liquid water mixing ratio including sub-gridscale liquid water." },

  { "qi_tot", 8, "1",
    "Gridbox-mean total ice water mixing ratio",
    "mass_fraction_of_total_cloud_ice_in_air",
    "Gridbox-mean total ice water mixing ratio including sub-gridscale ice." },

  { "ls_cloud_fraction", 8, "1",
    "Large scale cloud fraction",
    "large_scale_cloud_area_fraction",
    NULL },

  { "conv_cloud_fraction", 8, "1",
    "Convective cloud fraction",
    "convective_cloud_area_fraction",
    NULL },

  { "cloud_fraction", 8, "1",
    "Cloud fraction",
    "cloud_area_fraction",
    NULL },

  { "cloud_fraction_XuR", 8, "1",
    "Xu and Randall cloud fraction",
    "cloud_area_fraction_XuR",
    NULL },

  { "reff", 8, "m",
    "Diagnostic mean reff",
    "r_effective",
    "Diagnostic mean of reff_ice(T) and reff_water(T, lwc)"},

  { "flx_height", 3, "m",
    "Height above ground",
    NULL,
    NULL },

  { "flx_height_amsl", 3, "m",
    "Height above mean sea level",
    NULL,
    NULL },

  { "flx_net_sw", 8, "W m-2",
    "Net shortwave flux",
    "net_downward_shortwave_flux_in_air",
    NULL },

  { "flx_net_lw", 8, "W m-2",
    "Net longwave flux",
    "net_downward_longwave_flux_in_air",
    NULL },

  { "flx_down_sens_heat", 8, "W m-2",
    "Sensible heat flux",
    NULL,
    NULL },

  { "flx_turb_moist", 8, "kg m-2 s-1",
    "Turbulent moisture flux",
    NULL,
    NULL },

  { "flx_ls_rain", 8, "kg m-2 s-1",
    "Large-scale rainfall flux",
    "large_scale_rainfall_flux",
    NULL },

  { "flx_ls_snow", 8, "kg m-2 s-1",
    "Large-scale snowfall flux",
    "large_scale_snowfall_flux",
    NULL },

  { "flx_conv_rain", 8, "kg m-2 s-1",
    "Convective rainfall flux",
    "convective_rainfall_flux",
    NULL },

  { "flx_conv_snow", 8, "kg m-2 s-1",
    "Convective snowfall flux",
    "convective_snowfall_flux",
    NULL },

  { "flx_turb_mom_u", 8, "kg m-1 s-2",
    "Zonal turbulent momentum flux",
    "downward_eastward_momentum_flux_in_air",
    NULL },

  { "flx_turb_mom_v", 8, "kg m-1 s-2",
    "Meridional turbulent momentum",
    "downward_northward_momentum_flux_in_air",
    NULL },

  { "sfc_height_amsl", 3, "m",
    "Surface height above mean sea level",
    NULL,
    NULL },

  { "sfc_pressure", 0, "Pa",
    "Surface pressure",
    "surface_pressure",
    NULL },

  { "sfc_temperature", 2, "K",
    "Surface temperature",
    "surface_temperature",
    NULL },

  { "sfc_net_sw", 8, "W m-2",
    "Surface net downward shortwave flux",
    "surface_net_downward_shortwave_flux",
    NULL },

  { "sfc_net_lw", 8, "W m-2",
    "Surface net downward longwave flux",
    "surface_net_downward_longwave_flux",
    NULL },

  { "sfc_down_sw", 8, "W m-2",
    "Surface downwelling shortwave flux",
    "surface_downwelling_shortwave_flux",
    NULL },

  { "sfc_down_lw", 8, "W m-2",
    "Surface downwelling longwave flux",
    "surface_downwelling_longwave_flux",
    NULL },

  { "sfc_up_sw", 8, "W m-2",
    "Surface upwelling shortwave flux",
    "surface_upwelling_shortwave_flux",
    NULL },

  { "sfc_up_lw", 8, "W m-2",
    "Surface upwelling longwave flux",
    "surface_upwelling_longwave_flux",
    NULL },

  { "toa_up_sw", 8, "W m-2",
    "Top of atmosphere upwelling shortwave flux",
    "top_of_atmosphere_upwelling_shortwave_flux",
    NULL },

  { "toa_down_sw", 8, "W m-2",
    "Top of atmosphere downwelling shortwave flux",
    "top_of_atmosphere_downwelling_shortwave_flux",
    NULL },

  { "toa_up_lw", 8, "W m-2",
    "Top of atmosphere upwelling longwave flux",
    "top_of_atmosphere_upwelling_longwave_flux",
    NULL },

  { "sfc_cs_down_sw", 8, "W m-2",
    "Clear sky downwelling shortwave flux",
    "surface_downwelling_shortwave_flux_in_air_assuming_clear_sky",
    NULL },

  { "sfc_cs_down_lw", 8, "W m-2",
    "Clear sky downwelling longwave flux",
    "surface_downwelling_longwave_flux_in_air_assuming_clear_sky",
    NULL },

  { "sfc_down_lat_heat_flx", 8, "W m-2",
    "Latent heat flux",
    "surface_downward_latent_heat_flux",
    NULL },

  { "sfc_down_sens_heat_flx", 8, "W m-2",
    "Sensible heat flux",
    "surface_downward_sensible_heat_flux",
    NULL },

  { "sfc_ls_rain", 8, "kg m-2",
    "Large-scale rainfall amount",
    "large_scale_rainfall_amount",
    NULL },

  { "sfc_ls_rain_flx", 8, "kg m-2 s-1",
    "Surface large-scale rainfall flux",
    "large_scale_rainfall_flux",
    NULL },

  { "sfc_conv_rain", 8, "kg m-2",
    "Convective rainfall amount",
    "convective_rainfall_amount",
    NULL },

  { "sfc_conv_rain_flx", 8, "kg m-2 s-1",
    "Surface convective rainfall flux",
    "convective_rainfall_flux",
    NULL },

  { "sfc_ls_snow", 8, "kg m-2",
    "Large-scale snowfall amount",
    "large_scale_snowfall_amount",
    NULL },

  { "sfc_ls_snow_flx", 8, "kg m-2 s-1",
    "Surface large-scale snowfall flux",
    "large_scale_snowfall_flux",
    NULL },

  { "sfc_conv_snow", 8, "kg m-2",
    "Convective snowfall amount",
    "convective_snowfall_amount",
    NULL },

  { "sfc_conv_snow_flx", 8, "kg m-2 s-1",
    "Surface convective snowfall flux",
    "convective_snowfall_flux",
    NULL },

  { "sfc_ls_precip_fraction", 8, "1",
    "Large-scale precipitation fraction",
    NULL,
    NULL },

  { "sfc_ls_cloud_fraction", 8, "1",
    "Surface large-scale cloud fraction",
    NULL,
    NULL },

  { "sfc_conv_cloud_fraction", 8, "1",
    "Surface convective cloud fraction",
    NULL,
    NULL },

  { "sfc_cloud_fraction", 8, "1",
    "Surface total cloud fraction",
    NULL,
    NULL },

  { "sfc_bl_height", 3, "m",
    "Boundary layer height",
    NULL,
    NULL },

  { "sfc_albedo", 6, "1",
    "Surface albedo",
    "surface_albedo",
    NULL },

  { "sfc_temp_2m", 2, "K",
    "Temperature at 2m",
    NULL,
    NULL },

  { "sfc_q_2m", 8, "1",
    "Specific humidity at 2m",
    NULL,
    NULL },

  { "sfc_rough_mom", 6, "m",
    "Surface roughness for momentum",
    NULL,
    NULL },

  { "sfc_rough_heat", 6, "m",
    "Surface roughness for heat",
    NULL,
    NULL },

  { "sfc_skin_temp", 2, "K",
    "Skin temperature",
    NULL,
    NULL },

  { "sfc_wind_u_10m", 6, "m s-1",
    "Zonal wind at 10m",
    NULL,
    NULL },

  { "sfc_wind_v_10m", 6, "m s-1",
    "Meridional wind at 10m",
    NULL,
    NULL },

  { "sfc_geopotential", 6, "m2 s-2",
    "Geopotential",
    "geopotential",
    NULL },

  { "sfc_geopotential_height", 3, "m",
    "Geopotential height",
    "geopotential_height",
    NULL },

  { "gas_atten", 8, "dB",
    "Two-way attenuation from the ground due to atmospheric gases",
    NULL,
    NULL },

  { "specific_gas_atten", 8, "dB km-1",
    "Specific one-way attenuation due to atmospheric gases",
    NULL,
    NULL },

  { "specific_saturated_gas_atten", 8, "dB km-1",
    "Specific one-way attenuation due to atmospheric gases for saturated air (saturated with respect to ice below 0 degrees C)",
    NULL,
    NULL },

  { "specific_dry_gas_atten", 8, "dB km-1",
    "Specific one-way attenuation due to atmospheric gases for dry air (no water vapour)",
    NULL,
    NULL },

  { "K2", 8, "dB km-1",
    "Dielectric parameter (|K|^2) of liquid water",
    NULL,
    NULL },

  { "specific_liquid_atten", 8, "(dB km-1)/(g m-3)",
    "Specific one-way attenuation due to liquid water, per unit liquid water content",
    NULL,
    NULL }
};


/************************************************************************/
/************************************************************************/


char * ncTitles[CN_ML_COUNT] =
{
  "UK Met Office UM (Mesoscale) single-site output",
  "UK Met Office UM (Global) single-site output",
  "ECMWF single-site output",
  "Meteo-France ARPEGE single-site output",
  "KNMI HIRLAM single-site output",
  "KNMI RACMO single-site output",
  "SMHI RCA single-site output",
  "DWD LM single-site output",
  "UK Met Office UM (North Atlantic And European) single-site output",
  "UK Met Office UM (UK High Resolution) single-site output"
};


char * ncSources[CN_ML_COUNT] =
{
  "UK Met Office Unified Model (Mesoscale)",
  "UK Met Office Unified Model (Global)",
  "ECMWF Integrated Forecast System (IFS)",
  "Meteo-France ARPEGE",
  "KNMI High Resolution Limited Area Model (HIRLAM)",
  "KNMI Regional Atmospheric Climate Model (RACMO)",
  "SMHI Rossby Centre Regional Atmospheric Model, (RCA)",
  "DWD Lokal Modell, (LM)",
  "UK Met Office Unified Model (North Atlantic And European)",
  "UK Met Office Unified Model (UK High Resolution)"
};


char * ncModelInstitutions[CN_ML_COUNT] =
{
  "UK Met Office",
  "UK Met Office",
  "European Centre for Medium-Range Weather Forecasting",
  "Meteo-France",
  "KNMI",
  "KNMI",
  "SMHI",
  "DWD",
  "UK Met Office",
  "UK Met Office"
};


/************************************************************************/
/************************************************************************/


float ncHorizontalResolutions[CN_ML_COUNT][CN_SITE_COUNT] = 
{
  { 12.0F, 12.0F, 12.0F, 12.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F }, /* CN_ML_UKMO_UM_MESOSCALE */
  { 60.0F, 60.0F, 60.0F, 60.0F, 60.0F, 60.0F, 60.0F, 60.0F, 60.0F, 60.0F, 60.0F }, /* CN_ML_UKMO_UM_GLOBAL */
  { 40.0F, 40.0F, 40.0F, 40.0F, 40.0F, 40.0F, 40.0F, 40.0F, 40.0F, 40.0F, 40.0F }, /* CN_ML_ECMWF_IFS */
  { 23.4F, 23.4F, 23.2F, 23.5F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F, 25.0F, 25.0F }, /* CN_ML_METEO_FRANCE_ARPEGE*/
  {  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F }, /* CN_ML_KNMI_HIRLAM */
  { 18.0F, 18.0F, 18.0F, 18.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F, 18.0F, 18.0F }, /* CN_ML_KNMI_RACMO */
  { 44.0F, 44.0F, 44.0F, 44.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F, 44.0F, 44.0F }, /* CN_ML_SMHI_RCA */
  {  7.0F,  7.0F,  7.0F,  7.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  7.0F,  7.0F }, /* CN_ML_DWD_LM */
  { 12.0F, 12.0F, 12.0F, 12.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F }, /* CN_ML_UKMO_UM_NAE */
  {  4.0F,  4.0F,  4.0F,  4.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F,  0.0F }  /* CN_ML_UKMO_UM_UKHIGRES */
};

/************************************************************************/
/************************************************************************/


char * ncMakeFilename(int model, int time, int site)
{
  static char   filename[FILENAME_MAX + 1] = "";
  static char   siteName[100];
  static char * models[] = { "UKMO-UM-MESOSCALE", "UKMO-UM-GLOBAL",
    "ECMWF-IFS", "METEO-FRANCE-ARPEGE", "KNMI-HIRLAM", "KNMI-RACMO", "SMHI-RCA", "DWD-LM", "UKMO-UM-NAE", "UKMO-UM-UKHIGHRES"};

  int date;

  date = utDateFromMins(time);
  time = utTimeFromMins(time);
  strcpy(siteName, cnGetSiteName(site, 'n'));
  utLower(siteName);
  sprintf(filename, "%s_%08d_%04d_%s",
    models[model], date, time, siteName);

  return filename;
}


char * ncMakeTimeUnits(int time, int utcTime)
{
  static char units[100];
  int         date;
  int         year;
  int         month;
  int         day;
  int         hour;
  int         min;
  int         utcHour;
  int         utcMin;

  date = utDateFromMins(time);
  time = utTimeFromMins(time);
  utExplodeDate(date, &year, &month, &day);
  utExplodeTime(time, &hour, &min);
  utExplodeTime(utcTime, &utcHour, &utcMin);
  if ( utcMin < 0 )
  {
    utcMin -= utcMin;
  }

  sprintf(units, "hours since %04d-%02d-%02d %02d:%02d:00 %+03d:%02d",
    year, month, day, hour, min, utcHour, utcMin);

  return units;
}


char * ncMakeInitializationTime(int time, int utcTime)
{
  static char initTime[100];
  int         date;
  int         year;
  int         month;
  int         day;
  int         hour;
  int         min;
  int         utcHour;
  int         utcMin;

  date = utDateFromMins(time);
  time = utTimeFromMins(time);
  utExplodeDate(date, &year, &month, &day);
  utExplodeTime(time, &hour, &min);
  utExplodeTime(utcTime, &utcHour, &utcMin);
  if ( utcMin < 0 )
  {
    utcMin -= utcMin;
  }

  sprintf(initTime, "%04d-%02d-%02d %02d:%02d:00 %+03d:%02d",
    year, month, day, hour, min, utcHour, utcMin);

  return initTime;
}


char * ncMakeHistoryLine(char * user, int argc, char * argv[])
{
  enum
  {
    MAX_LEN_TIME = 32,
    MAX_LEN_HOST = 32
  };

  static char timeString[MAX_LEN_TIME + 1];
  static char hostString[MAX_LEN_HOST + 1];
  time_t      calTime;
  struct tm * localTime = NULL;
  char *      cmdLine   = utMakeCommandLine(argc, argv);


  calTime = time(NULL);
  if ( calTime != -1 )
  {
    localTime = localtime(&calTime);
  }

  if ( localTime != NULL )
  {
    strftime(timeString, MAX_LEN_TIME, "%Y-%m-%dT%H:%M %Z, ", localTime);
  }
  else
  {
    strcpy(timeString, "[time unavailable], ");
  }

  if ( user == NULL )
  {
    user = utGetUser();
  }

  gethostname(hostString, MAX_LEN_HOST - 1);

  return utAppendStr(utMakeStr(timeString), utMakeStr(user),
    utMakeStr(" on "), utMakeStr(hostString), utMakeStr(", "),
    cmdLine, NULL);
}


int ncSearchNames(char * name)
{
  int retValue = -1;

  for ( retValue = CN_VARIABLE_ID_COUNT - 1;
    retValue >= 0 && strcmp(ncVariables[retValue].name, name) != 0;
    retValue-- )
  {
  }
  return retValue;
}


/************************************************************************/
/************************************************************************/
