

#include "cnutils.h"


/************************************************************************/
/************************************************************************/


static int utDayInYear[13] =
{ 0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };


/* Minutes in a day. */
static const int utMinsPerDay = 24 * 60;


/************************************************************************/
/************************************************************************/


char * utMakeStr(char * s)
{
  char * retValue = NULL;

  if ( s != NULL )
  {
    retValue = (char *) malloc(strlen(s) + 1);
    if ( retValue != NULL )
    {
      strcpy(retValue, s);
    }
  }
  return retValue;
}


char * utAppendStrV(char * s, va_list ap)
{
  char *  s2;
  char *  retValue = NULL;
  int     l        = ( s == NULL ) ? 0 : strlen(s);
  va_list temp     = ap;

  /* First get the total length of the new string. */
  s2 = va_arg(ap, char *);
  while ( s2 != NULL )
  {
    l += strlen(s2);
    s2 = va_arg(ap, char *);
  }

  retValue = (char *) malloc(l + 1);
  if ( retValue != NULL )
  {
    if ( s != NULL )
    {
      strcpy(retValue, s);
      free(s);
    }
    else
    {
      retValue[0] = '\0';
    }

    ap = temp;
    s2 = va_arg(ap, char *);
    while ( s2 != NULL )
    {
      strcat(retValue, s2);
      free(s2);
      s2 = va_arg(ap, char *);
    }
  }

  return retValue;
}


char * utAppendStr(char * s, ...)
{
  va_list ap;
  char *  retValue = NULL;

  va_start(ap, s);
  retValue = utAppendStrV(s, ap);
  va_end(ap);
  return retValue;
}


char * utUpper(char * s)
{
  if ( s != NULL )
  {
    char * ss;
    for ( ss = s; *ss != '\0'; ss++ )
    {
      *ss = (char) toupper((int) *ss);
    }
  }
  return s;
}


char * utLower(char * s)
{
  if ( s != NULL )
  {
    char * ss;
    for ( ss = s; *ss != '\0'; ss++ )
    {
      *ss = (char) tolower((int) *ss);
    }
  }
  return s;
}


char * utMixCase(char * s)
{
  if ( s != NULL )  
  {
    int i;
    int first;
    int l;

    l     = strlen(s);
    first = 1;
    for ( i = 0; i < l; i++ )
    {
      if ( !isalpha((int) s[i]) )
      {
        first = 1;
      }
      else
      if ( first )
      {
        s[i]  = (char) toupper((int) s[i]);
        first = 0;
      }
      else
      {
        s[i]  = (char) tolower((int) s[i]);
      }
    }
  }

  return s;
}


char * utGetUser(void)
{
  char * retValue;

  retValue = getenv("LOGNAME");
  if ( retValue == NULL )
  {
    retValue = getenv("USER");
    if ( retValue == NULL )
    {
      retValue = "[unknown user]";
    }
  }
  return retValue;
}


char * utMakeCommandLine(int argc, char * argv[])
{
  char * retValue = NULL;
  int    i;

  for ( i = 0; i < argc; i++ )
  {
    char * s = utMakeStr(argv[i]);

    /* Check if there are any spaces/tabs in the argument.
     * If there is then need quotes around the argument.
     */
/*
    if ( s != NULL && strpbrk(s, " \t") != NULL )
    {
      s = utAppendStr(utMakeStr("\""), s, utMakeStr("\""), NULL);
    }
 */
    if ( s != NULL && i < (argc - 1) )
    {
      /* Append a space to the end of the argument. */
      s = utAppendStr(s, utMakeStr(" "), NULL);
    }

    if ( s != NULL )
    {
      retValue = utAppendStr(retValue, s, NULL);
    }
  }

  return retValue;
}


int utMakeDate(int year, int month, int day)
{
  return year * 10000 + month * 100 + day;
}


void utExplodeDate(int date, int * year, int * month, int * day)
{
assert(year != NULL);
assert(month != NULL);
assert(day != NULL);

  *year  = date / 10000;
  date  %= 10000;
  *month = date / 100;
  *day   = date % 100;
}


/*
 * POST: returns 1 if 'year' is a leap year, 0 otherwise.
 */
#define utIsLeapYear(year)  (( ((year) % 4) == 0 ) ? 1 : 0)


/*
 * POST: returns the number of leap years between 1960 and year.
 */
#define utNumLeapYears(year) \
(( (year) > 1960 ) ? (((year) - 1 - 1960) / 4) + 1 : 0)


/*
 * POST: returns the number of days between 1960 and the start of
 * 'year'.
 */
#define utNumDaysStartYear(year) \
(((year) - 1960) * 365 + utNumLeapYears(year))


/*
 * POST: returns the number of days to the start of the month.
 */
#define utNumDaysStartMonth(year, month) \
(utDayInYear[month] + (( (month) > 2 ) ? utIsLeapYear(year) : 0))


/* Calculate the number of days since 1960.
 *
 * PRE: 'date' is of the format YYYYMMDD.
 *
 * POST: returns the number of days since 1960 represented by 'date'.
 * Jan 1 1960 is day 0.
 * Returns -1 if date is before 1960.
 */
int utDaysSince1960(int date);


/* Get the date from the number of days since 1960.
 *
 * PRE: days>=0;
 * 'days' is the number of days since 1960.
 *
 * POST: returns date in the format YYYYMMDD.
 */
int utDateFromDays(int days);


/* Converts number of minutes to format HHMM.
 * POST: returns the time in the format HHMM.
 */
int utFromMins(int mins);


int utDaysSince1960(int date)
{
  int year;
  int month;
  int day;
  int retValue = -1;

  utExplodeDate(date, &year, &month, &day);
  if ( year >= 1960 )
  {
    retValue = utNumDaysStartYear(year) + utNumDaysStartMonth(year, month)
      + (day - 1);
  }

  return retValue;
}


int utDateFromDays(int days)
{
  int year;
  int month;
  int day;
  int numDays;


assert(days >= 0);


  /* First guess at the year. */
  year    = 1960 + days / 365;
  numDays = utNumDaysStartYear(year);
  if ( numDays > days )
  {
    year--;
    numDays = utNumDaysStartYear(year);
  }
  days -= numDays;

  /* First guess at the month. */
  month = days / 28 + 1;
  if ( month > 12 )
  {
    month = 12;
  }
  numDays = utNumDaysStartMonth(year, month);
  if ( numDays > days )
  {
    month--;
    numDays = utNumDaysStartMonth(year, month);
  }
  day = days - numDays + 1;

  return utMakeDate(year, month, day);
}


int utMakeTime(int hour, int min)
{
  return hour * 100 + min;
}


void utExplodeTime(int time, int * hour, int * min)
{
assert(hour != NULL);
assert(min != NULL);

  *hour = time / 100;
  *min  = time % 100;
}


int utFromDecTime(double time)
{
  double ip;
  int    min  = (int) (modf(time, &ip) * 60.0);
  int    hour = (int) ip;
  return utMakeTime(hour, min);
}


/* 'time' is in format HHMM.
 *
 * POST: returns the decimal number of hours.
 */
double utToDecTime(int time)
{
  return (double) (utTimeInMins(time)) / 60.0;
}


int utTimeInMins(int time)
{
  int hour;
  int min;
  utExplodeTime(time, &hour, &min);
  return hour * 60 + min;
}


int utFromMins(int mins)
{
  return utMakeTime(mins / 60, mins % 60);
}


int utToMins(int date, int time)
{
  int days = utDaysSince1960(date);
  if ( days < 0 )
  {
    return -1;
  }
  return days * utMinsPerDay + utTimeInMins(time);
}


int utDateFromMins(int mins)
{
  return utDateFromDays(mins / utMinsPerDay);
}


int utTimeFromMins(int mins)
{
  return utFromMins(mins % utMinsPerDay);
}


/************************************************************************/
/************************************************************************/

