/*
 * Some utilities.
 *
 * HISTORY
 * 2003-04-02 ao started.
 * 2003-04-15 ao added the 'utFromDecTime()' function.
 * 2003-07-31 ao removed 'utMakeCommandLine()' from header and modified
 *   'utMakeHistoryLine()' to include command line.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNUTILS_H)
#define CNUTILS_H


/************************************************************************/
/************************************************************************/


#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <math.h>


/************************************************************************/
/************************************************************************/


/* Allocates memory and copies 's'.
 *
 * POST: returns NULL if 'malloc()' failed or s==NULL.
 */
char * utMakeStr(char * s);


/* Append strings to end of 's'.
 * After 's' there must be a NULL terminated list of (char *).
 * Each string, including 's' MUST be 'malloc()' allocated.
 * 's' can be NULL followed by non-NULL strings terminated by NULL.
 *
 * POST: returns NULL if 'malloc()' failed and all the strings are
 * left unaltered, otherwise returns a newly allocated string containing
 * the concatenation of all the arguments and all the arguments will
 * be freed.
 */
char * utAppendStr(char * s, ...);


/* Same as 'utAppendStr()' except uses the 'va_list' argument. */
char * utAppendStrV(char * s, va_list ap);


/* Convert all letters in 's' to upper case.
 * POST: returns 's'.
 */
char * utUpper(char * s);


/* Convert all letters in 's' to lower case.
 * POST: returns 's'.
 */
char * utLower(char * s);


/* Capitalises each separate word in 's' and makes other letters lower
 * case.
 * POST: returns 's';
 */
char * utMixCase(char * s);


/* Get the user name.
 *
 * POST: returns the user name or "[unknown user]".
 */
char * utGetUser(void);


/*
 * POST: returns a newly created string with the command line arguments
 * separated by spaces.
 * Returns NULL if 'malloc()' failed.
 */
char * utMakeCommandLine(int argc, char * argv[]);


/* 'time' is the decimal number of hours.
 *
 * POST: returns time in format HHMM.
 */
int utFromDecTime(double time);


/* 'time' is in format HHMM.
 *
 * POST: returns the decimal number of hours.
 */
double utToDecTime(int time);


/* 'time' is in the format HHMM.
 *
 * POST: returns the number of minutes represented by 'time'.
 */
int utTimeInMins(int time);


/* Convert 'date' and 'time' to minutes from 1960.
 *
 * PRE: 'date' is in the format YYYYMMDD and 'time' is in the format
 * HHMM.
 *
 * POST: returns -1 if year<1960 otherwise returns the number of minutes
 * from the start of 1960 represented by 'date' and 'time'.
 */
int utToMins(int date, int time);


/*
 * PRE: 'mins' is the number of minutes from 1960.
 *
 * POST: returns the date in the format YYYYMMDD.
 */
int utDateFromMins(int mins);


/*
 * PRE: 'mins' is the number of minutes from 1960.
 *
 * POST: returns the time in the format HHMM.
 */
int utTimeFromMins(int mins);


/*
 * POST: returns the date in the format YYYYMMDD.
 */
int utMakeDate(int year, int month, int day);


/*
 * PRE: 'date' is in the format YYYYMMDD.
 *
 * POST: extracts the year, month, day from 'date'.
 */
void utExplodeDate(int date, int * year, int * month, int * day);


/*
 * POST: returns the time in the format HHMM.
 */
int utMakeTime(int hour, int min);


/*
 * POST: extracts the hour and minutes from 'time'.
 */
void utExplodeTime(int time, int * hour, int * min);


/************************************************************************/
/************************************************************************/


#endif
