

enum
{
  MAXLINELEN = 4096,

  /* The number of lines from the start of the file to read.
   * This should be enough to determine the information required.
   */
  NUMLINES   = 20,

  /* Extra surface file types for ECIFS. */
  FT_SFC2    = CN_FT_COUNT,
  FT_SFCE
};


typedef struct
{
  int model;
  int fileType;
  int site;
  int initTime;
  int forecastTime;
} cnTFileInfo;


void getFileInfo(char * filename, cnTFileInfo * info)
{
#define FINDSTR(line,s) p=strstr(line,s); if ( p!=NULL ) p+=strlen(s)


  static char hdr[NUMLINES][MAXLINELEN + 1];

  FILE * infile;
  int    i;

  /**********************************************************************/

  info->model        = -1;
  info->fileType     = -1;
  info->site         = -1;
  info->initTime     = -1;
  info->forecastTime = -1;

  /**********************************************************************/

  infile = fopen(filename, "r");
  if ( infile == NULL )
  {
    printError(PE_EXIT, "unable to open file: %s", filename);
  }

  /**********************************************************************/

  for ( i = 0; i < NUMLINES; i++ )
  {
    if ( fgets(hdr[i], MAXLINELEN, infile) == NULL )
    {
      if ( ferror(infile) != 0 )
      {
        printError(PE_EXIT, "error reading from file: %s", filename);
      }
      else
      {
        printError(PE_EXIT, "unexpected EOF in: %s at line: %d",
          filename, i);
      }
    }

    utUpper(hdr[i]);
  }

  /**********************************************************************/

  if ( hdr[0][0] == '%' ) /* MOUMM */
  {
    static char * varLine = "%YYYY MM DD HH LEV   H(KM)";
    static char * sfcLine = "%YYYY MM DD HH      OROG (M)";

    char * p;



    /* Check the filename to determine which model */
    /* Default is mesoscale */
    info->model = CN_ML_MOUMM;

    FINDSTR(filename, "NAE");
    if ( p != NULL ) {
      info->model = CN_ML_MOUNA;
    }
    else {
      FINDSTR(filename, "HIGHRES");
      if ( p != NULL ) {
	info->model = CN_ML_MOUHR;
      }
    }
    
    /* Test the second line. */
    if ( strncmp(hdr[1], varLine, strlen(varLine)) == 0 )
    {
      info->fileType = CN_FT_VAR;
    }
    else
    if ( strncmp(hdr[1], sfcLine, strlen(sfcLine)) == 0 )
    {
      info->fileType = CN_FT_SFC;
    }

    FINDSTR(hdr[0], "STATION ");
    if ( p != NULL )
    {
      int siteId = atoi(p);

      FINDSTR(p, "FROM ");
      if ( p != NULL )
      {
        int year;
        int month;
        int day;
        int hour;
        int min;

        sscanf(p, "%d %d %d %d %d", &year, &month, &day, &hour, &min);

        info->initTime     = utToMins(year * 10000 + month * 100 + day,
          hour * 100 + min);
        info->forecastTime = info->initTime;

        if ( info->initTime >= utToMins(20021101, 0000) )
        {
          switch ( siteId )
          {
            case 200101: info->site = CN_SITE_AB; break;
            case 200110: info->site = CN_SITE_CH; break;
            case 200214: info->site = CN_SITE_PA; break;
            case 200215: info->site = CN_SITE_CA; break;
            case 200216: info->site = CN_SITE_CH; break;
            default: break;
          }
        }
        else
        {
          switch ( siteId )
          {
            case 200101: info->site = CN_SITE_AB; break;
            case 200110: info->site = CN_SITE_CH; break;
            case 200214: info->site = CN_SITE_PA; break;
            case 200215: info->site = CN_SITE_CA; break;
            case 200216: info->site = CN_SITE_CH; break;
            default: break;
          }
        }
      }
    }
  }

  else /*****************************************************************/

  if ( hdr[0][0] == '#' ) /* ECIFS, MFARP, KNRAC, DWDLM */
  {
    /* Count the number of header lines. */
    int numHdrLines = 0;
    for ( i = 0; i < NUMLINES && hdr[i][0] == '#'; i++ )
    {
      numHdrLines++;
    }
    /********************************************************************/

    if ( numHdrLines == 3 ) /* ECIFS */
    {
      static char * varLine  = "#   IDAT ITIM     VDAT  VTIM  LEV    P  HIST";
      static char * flxLine  = "#   IDAT ITIM     VDAT  VTIM  LEV    T  FSWR";
      static char * sfcLine  = "#   IDAT ITIM     VDAT  VTIM  LEV    SFSSLHF";
      static char * sfc2Line = "#   IDAT ITIM     VDAT  VTIM  LEV    SFSLWDO";
      static char * sfceLine = "#   IDAT ITIM     VDAT  VTIM  LEV    SFST2M";

      float dummy;
      char dummystr[4];
      int   siteId;
      int   idate;
      float itime;
      int   vdate;
      float vtime;

      info->model = CN_ML_ECIFS;
      if ( strncmp(hdr[2], varLine, strlen(varLine)) == 0 )
      {
        info->fileType = CN_FT_VAR;
      }
      else
      if ( strncmp(hdr[2], flxLine, strlen(flxLine)) == 0 )
      {
        info->fileType = CN_FT_FLX;
      }
      else
      if ( strncmp(hdr[2], sfcLine, strlen(sfcLine)) == 0 )
      {
        info->fileType = CN_FT_SFC;
      }
      else
      if ( strncmp(hdr[2], sfc2Line, strlen(sfc2Line)) == 0 )
      {
        info->fileType = FT_SFC2;
      }
      else
      if ( strncmp(hdr[2], sfceLine, strlen(sfceLine)) == 0 )
      {
        info->fileType = FT_SFCE;
      }


      sscanf(hdr[1], "# %s %f %d %f %f %f",
        dummystr, &dummy, &siteId, &dummy, &dummy, &dummy);

      switch ( siteId )
      {
        case  4: info->site = CN_SITE_T1; break;
        case  6: info->site = CN_SITE_T3; break;
        case  7: info->site = CN_SITE_PO; break;
        case 13: info->site = CN_SITE_NS; break;
        case 14: info->site = CN_SITE_NS; break;
        case 20: info->site = CN_SITE_SG; break;
        case 21: info->site = CN_SITE_CH; break;
        case 22: info->site = CN_SITE_LI; break;
        case 30: info->site = CN_SITE_CA; break;
        case 47: info->site = CN_SITE_PA; break;
        case 48: info->site = CN_SITE_T2; break;
        case 76: info->site = CN_SITE_NI; break;
        default: break;
      }

      sscanf(hdr[3], "%d %f %d %f", &idate, &itime, &vdate, &vtime);

      info->initTime     = utToMins(idate, utFromDecTime(itime));
      info->forecastTime = utToMins(vdate, utFromDecTime(vtime));
    }

    else /***************************************************************/

    if ( numHdrLines == 4 ) /* MFARP, DWDLM */
    {

      if ( strstr(hdr[0], "STATION") != NULL ) /* MFARP */
      {
	static char * varLine = "# IDAT    ITIM   VDAT   VTIME  LEV    H LEVEL      P  HIST      U  HIST";
	static char * flxLine = "# IDAT    ITIM   VDAT   VTIME  LEV    H LEVEL      T  FSWR      T  FLWR";
	static char * sfcLine = "# IDAT    ITIM   VDAT   VTIME  LEV    SFSSLHF      SFSSSHF";
	static char * sfcLine1= "# IDAT    ITIM   VDAT   VTIME LEV     SFSSLHF      SFSSSHF";
	
	int    idate;
	float  itime;
	int    vdate;
	float  vtime;
	char * p;
	
	info->model = CN_ML_MFARP;
	
	if ( strncmp(hdr[3], varLine, strlen(varLine)) == 0 )
	  {
	    info->fileType = CN_FT_VAR;
	  }
	else if ( strncmp(hdr[3], flxLine, strlen(flxLine)) == 0 )
	  {
	    info->fileType = CN_FT_FLX;
	  }
	else if ( strncmp(hdr[3], sfcLine, strlen(sfcLine)) == 0 || strncmp(hdr[3], sfcLine1, strlen(sfcLine)) == 0)
	  {
	    info->fileType = CN_FT_SFC;
	  }
	
	FINDSTR(hdr[0], "NAME: ");
	if ( strstr(p, "CHILBOLTON") != NULL )
	  {
	    info->site = CN_SITE_CH;
	  }
	else if ( strstr(p, "CABAUW") != NULL )
	  {
	    info->site = CN_SITE_CA;
	  }
	else if ( strstr(p, "PALAISEAU") != NULL )
	  {
	    info->site = CN_SITE_PA;
	  }
	else if ( strstr(p, "LINDENBERG") != NULL )
	  {
	    info->site = CN_SITE_LI;
	  }
	else if ( strstr(p, "POTENZA") != NULL )
	  {
	    info->site = CN_SITE_PO;
	  }
	else if ( strstr(p, "SGP") != NULL )
	  {
	    info->site = CN_SITE_SG;
	  }
	else if ( strstr(p, "NSA") != NULL )
	  {
	    info->site = CN_SITE_NS;
	  }
	else if ( strstr(p, "NAURU") != NULL )
	  {
	    info->site = CN_SITE_T1;
	  }
	else if ( strstr(p, "MANUS") != NULL )
	  {
	    info->site = CN_SITE_T2;
	  }
	else if ( strstr(p, "DARWIN") != NULL )
	  {
	    info->site = CN_SITE_T3;
	  }
	
	sscanf(hdr[4], "%d %f %d %f", &idate, &itime, &vdate, &vtime);
	
	info->initTime     = utToMins(idate, utFromDecTime(itime));
	info->forecastTime = utToMins(vdate, utFromDecTime(vtime));
      }

      else /* DWDLM */
	{
	  static char * varLine = "# VTIME  LEV       HEIGHT       H HIST       P HIST       U HIST";
	  static char * sfcLine = "# VTIME       S HIST    SMSL HIST        SF10U        SF10V        SFT2M      SFTAU2M";
	  	
	  float dummy;
	  char  dummystr[3];
	   int   year;
	   int   month;
	   int   day;
	   int   hour;
	   int   min;
	  int  siteId;

	  info->model = CN_ML_DWDLM;
	  
	  if ( strncmp(hdr[3], varLine, strlen(varLine)) == 0 )
	    {
	      info->fileType = CN_FT_VAR;
	    }
	  else if ( strncmp(hdr[3], sfcLine, strlen(sfcLine)) == 0)
	    {
	      info->fileType = CN_FT_SFC;
	    }
	  
	  sscanf(hdr[0], "# %s %04d%02d%02d%02d%02d", dummystr, &year, &month, &day, &hour, &min);
	  info->initTime     = utToMins(year * 10000 + month * 100 + day,
					hour * 100 + min);
	  info->forecastTime = info->initTime;
	  
	  
	  sscanf(hdr[2], "# %d %f %f %f",
		 &siteId, &dummy, &dummy, &dummy);
	  
	  switch ( siteId )
	    {
	    case  1899: info->site = CN_SITE_PO; break;
	    case 44318: info->site = CN_SITE_PA; break;
	    case 58586: info->site = CN_SITE_CH; break;
	    case 61667: info->site = CN_SITE_LI; break;
	    case 60276: info->site = CN_SITE_CA; break;
	    case 40193: info->site = CN_SITE_MU; break;
	    case 45034: info->site = CN_SITE_KA; break;
	    case  2939: info->site = CN_SITE_PO; break;
	    case 46743: info->site = CN_SITE_PA; break;
	    case 61030: info->site = CN_SITE_CH; break;
	    case 64514: info->site = CN_SITE_LI; break;
	    case 63092: info->site = CN_SITE_CA; break;
	    case 42513: info->site = CN_SITE_MU; break;
	    case 47475: info->site = CN_SITE_KA; break;
	    default: break;
	    }
	  
	}
    }

    else /***************************************************************/
      
      if ( numHdrLines == 11 || numHdrLines == 14 ) /* KNRAC SMRCA*/
	{
	  int    date;
	  int    time;
	  char * p;
	  
	  
      if ( strstr(hdr[0], "SMHI") != NULL )
      {
        info->model = CN_ML_SMRCA;
	info->fileType = ( strstr(hdr[10], "LEV") != NULL || strstr(hdr[10], "lev") != NULL) ? CN_FT_VAR : CN_FT_SFC;
	if (strstr(hdr[10], "TNLEV") != NULL || strstr(hdr[10], "Tnlev") != NULL)
	  info->fileType = CN_FT_SFC;
      }

      else
      {
	info->model = CN_ML_KNRAC;
	info->fileType = ( numHdrLines == 11 ) ? CN_FT_VAR : CN_FT_SFC;
      }

      p = &hdr[4][1];
      sscanf(p, "%d %d", &date, &time);
      info->initTime = utToMins(date, time);

      if ( strstr(hdr[5], "CH") != NULL )
      {
        info->site = CN_SITE_CH;
      }
      else
      if ( strstr(hdr[5], "CA") != NULL )
      {
        info->site = CN_SITE_CA;
      }
      else
      if ( strstr(hdr[5], "LI") != NULL )
      {
        info->site = CN_SITE_LI;
      }
      else
      if ( strstr(hdr[5], "PA") != NULL )
      {
        info->site = CN_SITE_PA;
      }
      if ( strstr(hdr[5], "PO") != NULL )
      {
        info->site = CN_SITE_PO;
      }
      else

      sscanf(hdr[numHdrLines], "%d %d", &date, &time);
      info->forecastTime = utToMins(date, time);
    }
  }

  else /*****************************************************************/

  if ( strstr(hdr[1], "MOLTS") != 0 )  /* MOUMG */
  {
    int date;
    int time;

    /* Contains both surface and level variables so can't give
     * fileType.
     */
    info->model = CN_ML_MOUMG;

    sscanf(hdr[2], "INITIAL DATA TIME %d %d", &date, &time);
    info->initTime     = utToMins(date, time);
    info->forecastTime = info->initTime;

    if ( strstr(&hdr[4][36], "ABERYSTWYTH") != NULL )
    {
      info->site = CN_SITE_AB;
    }
    else
    if ( strstr(&hdr[4][36], "CHILBOLTON") != NULL )
    {
      info->site = CN_SITE_CH;
    }
    else
    if ( strstr(&hdr[4][36], "CABAUW") != NULL )
    {
      info->site = CN_SITE_CA;
    }
    else
    if ( strstr(&hdr[4][36], "PALAISEAU") != NULL )
    {
      info->site = CN_SITE_PA;
    }
    else
    if ( strstr(&hdr[4][36], "PARIS") != NULL )
    {
      info->site = CN_SITE_PA;
    }
    else
    if ( strstr(&hdr[4][36], "LINDENBERG") != NULL )
    {
      info->site = CN_SITE_LI;
    }
    else
    if ( strstr(&hdr[4][36], "POTENZA") != NULL )
    {
      info->site = CN_SITE_PO;
    }
    else
    if ( strstr(&hdr[4][36], "SOUTHERN") != NULL )
    {
      info->site = CN_SITE_SG;
    }
    else
    if ( strstr(&hdr[4][36], "NORTH") != NULL )
    {
      info->site = CN_SITE_NS;
    }
    else
    if ( strstr(&hdr[4][36], "NAURU") != NULL )
    {
      info->site = CN_SITE_T1;
    }
    else
    if ( strstr(&hdr[4][36], "MANUS") != NULL )
    {
      info->site = CN_SITE_T2;
    }
    else
    if ( strstr(&hdr[4][36], "DARWIN") != NULL )
    {
      info->site = CN_SITE_T3;
    }  
    if ( strstr(&hdr[4][36], "NIAMEY") != NULL )
    {
      info->site = CN_SITE_NI;
    }
  }

  /**********************************************************************/

  fclose(infile);
}
