

#include "cndwdlm.h"


/************************************************************************/
/************************************************************************/


/* PRE: 'siteId' is the site ID from the DWD LM model output file header.
 * POST: returns one of 'cnESites' from 'cnsites.h' or -1 if 'siteId'
 * is unrecognized.
 */
static int cnDWDLMGetSite(int siteId)
{
  int retValue = -1;

  switch ( siteId )
    {
    case  1899: retValue = CN_SITE_PO; break;
    case 44318: retValue = CN_SITE_PA; break;
    case 58586: retValue = CN_SITE_CH; break;
    case 61667: retValue = CN_SITE_LI; break;
    case 60276: retValue = CN_SITE_CA; break;
    case 40193: retValue = CN_SITE_MU; break;
    case 45034: retValue = CN_SITE_KA; break;
    case  2939: retValue = CN_SITE_PO; break;
    case 46743: retValue = CN_SITE_PA; break;
    case 61030: retValue = CN_SITE_CH; break;
    case 64514: retValue = CN_SITE_LI; break;
    case 63092: retValue = CN_SITE_CA; break;
    case 42513: retValue = CN_SITE_MU; break;
    case 47475: retValue = CN_SITE_KA; break;
    default: break;
    }
  
  return retValue;
}


void cnDWDLMGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTDWDLMheader * hdr/*OUT*/)
{
  int   siteId;
  char  modelconf[3];
  int   year;
  int   month;
  int   day;
  int   hour;
  int   min;

  /* The file header should be similar to:   
  #  LM1   200401130000
  #      gpt      lat      lon   height      lsm   levels     tnum    tstep  vvstart    vvend
  #    60276   51.947    4.949      0.1      0.9       35       48     3600        1       48
  # vtime  lev       height       H hist       P hist       U hist       V hist       T hist ...
  *
  * Both first and third lines contain information. 
  * First line: model configuration indicator (LM1/LME/LMK) 
  * and reference date.
  * Third line: The first number is the site number, 
  * the next two numbers are the latitude and longitude for
  * the site.
  */
  fgets(line, max, infile);  /* First line of header. */

  sscanf(line, "# %s %4d%2d%2d%2d%2d",
    modelconf, &year, &month, &day, &hour, &min);

  hdr->initTime = utToMins(year * 10000 + month * 100 + day,
    hour * 100 + min);

  fgets(line, max, infile);  /* Skip second line. */
  fgets(line, max, infile);  /* The third line. */

  sscanf(line, "# %d %f %f %f %f",
    &siteId, &hdr->lat, &hdr->lon, &hdr->sfcHeight, &hdr->landSeaFrac);

  hdr->site = cnDWDLMGetSite(siteId);
  if ( hdr->site < 0 )
  {
    printError(PE_EXIT, "%s: unrecognized site ID from header: %d\n",
     filename, siteId);
  }

  fgets(line, max, infile);  /* Skip fourth line of header. */
}


void cnDWDLMGetVarLine(char * line, cnTDWDLMvarLine * ln, cnTDWDLMheader * hdr)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  float vtime;

  sscanf(line, "%f %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &vtime,
    &ln->level,
    &ln->height,
    &ln->geopotential,
    &ln->pressure,
    &ln->uwind,
    &ln->vwind,
    &ln->temp,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->qr,
    &ln->qs,
    &ln->totql,
    &ln->totqi,
    &ln->cloudFrac,
    &ln->rh,
    &ln->wwind
  );

  ln->rh *= 0.01F;
  ln->cloudFrac *= 0.01F;

  ln->verTime  = hdr->initTime + vtime * 60;
}



void cnDWDLMGetSfcLine(char * line, cnTDWDLMsfcLine * ln, cnTDWDLMheader * hdr)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  float vtime;

  sscanf(line, "%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f"
    " %f %f %f %f %f %f %f %f %f %f",
    &vtime,
    &ln->pressure,
    &ln->pressure_msl,
    &ln->uWind10m,
    &ln->vWind10m,
    &ln->temp2m,
    &ln->dptemp2m,
    &ln->skinTemp,
    &ln->totRain,
    &ln->largeRain,
    &ln->largeSnow,
    &ln->convRain,
    &ln->convSnow,
    &ln->totCloud,
    &ln->CloudHigh,
    &ln->CloudMed,
    &ln->CloudLow,
    &ln->convCloudBase,
    &ln->convCloudTop,
    &ln->uTurbMomFlx,
    &ln->vTurbMomFlx,
    &ln->latHeatFlx,
    &ln->sensHeatFlx,
    &ln->netSwFlx,
    &ln->netLwFlx
  );

  ln->totCloud *= 0.01F;
  ln->verTime  = hdr->initTime + vtime * 60;
}


/************************************************************************/
/************************************************************************/
