/*
 * CloudNET Model Conversion Software
 *
 * Stuff for DWD Lokall Model output files.
 *
 * HISTORY
 * 2005-05-11 eoc started.
 *
 * AUTHORS
 * Ewan O'Connor
 */


#if !defined(CNDWDLM_H)
#define CNDWDLM_H


/************************************************************************/
/************************************************************************/


#include "cnapp.h"
#include "cnsites.h"
#include "cnutils.h"


/************************************************************************/
/************************************************************************/


/* The information from the file header. */
typedef struct
{
  int   site;  /* site ID as in 'cnsites.h'. */

  int   initTime;   /* initialization time, minutes since 1960. */

  /* model lon/lat coordinates. */
  float lon;
  float lat;

  float sfcHeight;
  float landSeaFrac;
} cnTDWDLMheader;


/* The information from each line of data from a 'var' file. */
typedef struct
{
  int   verTime;    /* minutes since 1960. */
  int   level;
  float height;
  float geopotential;
  float pressure;
  float uwind;
  float vwind;
  float temp;
  float q;
  float ql;
  float qi;
  float qr;
  float qs;
  float totql;
  float totqi;
  float cloudFrac;
  float rh;
  float wwind;
} cnTDWDLMvarLine;


/* The information from each line of data from a 'sfc' file. */
typedef struct
{
  int   initTime;   /* minutes since 1960. */
  int   verTime;    /* minutes since 1960. */
  float pressure;
  float pressure_msl;
  float uWind10m;
  float vWind10m;
  float temp2m;
  float dptemp2m;
  float skinTemp;
  float totRain;
  float largeRain;
  float largeSnow;
  float convRain;
  float convSnow;
  float totCloud;
  float CloudHigh;
  float CloudMed;
  float CloudLow;
  float convCloudBase;
  float convCloudTop;
  float uTurbMomFlx;
  float vTurbMomFlx;
  float latHeatFlx;
  float sensHeatFlx;
  float netSwFlx;
  float netLwFlx;
} cnTDWDLMsfcLine;


/************************************************************************/
/************************************************************************/


/* Read the first three lines from 'infile'.
 * Use 'line' as the buffer to read the lines into.
 * 'max' is the maximum length any line can be.
 */
void cnDWDLMGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTDWDLMheader * hdr/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnDWDLMGetVarLine(char * line, cnTDWDLMvarLine * ln/*OUT*/, cnTDWDLMheader * hdr);


/* 'line' is a line read from a 'sfc' file. */
void cnDWDLMGetSfcLine(char * line, cnTDWDLMsfcLine * ln/*OUT*/, cnTDWDLMheader * hdr);


/************************************************************************/
/************************************************************************/


#endif
