

#include "cnecifs.h"


/************************************************************************/
/************************************************************************/


/* PRE: 'siteId' is the site ID from the ECMWF model output file header.
 * POST: returns one of 'cnESites' from 'cnsites.h' or -1 if 'siteId'
 * is unrecognized.
 */
static int cnECIFSGetSite(int siteId)
{
  int retValue = -1;

  switch ( siteId )
  {
    case  4: retValue = CN_SITE_T1; break;
    case  6: retValue = CN_SITE_T3; break;
    case  7: retValue = CN_SITE_PO; break;
    case 13: retValue = CN_SITE_NS; break;
    case 14: retValue = CN_SITE_NS; break;
    case 20: retValue = CN_SITE_SG; break;
    case 21: retValue = CN_SITE_CH; break;
    case 22: retValue = CN_SITE_LI; break;
    case 30: retValue = CN_SITE_CA; break;
    case 47: retValue = CN_SITE_PA; break;
    case 48: retValue = CN_SITE_T2; break;
    case 76: retValue = CN_SITE_NI; break;
    default: break;
  }

  return retValue;
}


void cnECIFSGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTECIFSheader * hdr/*OUT*/)
{
  int   siteId;
  float dummy;
  char dummystr[4];

  /* The file header should be similar to:
 #  exp vp  i ty  west  north   east  south  west2  north2  east2 south2
 # 0001  1 21  4 358.40  51.45
 #   idat itim     vdat  vtim  lev    P  HIST ...
  *
  * Only the second line is of interest. The first two numbers are
  * ignored, the third number is the site number, the fourth number is
  * ignored and the last two numbers are the longitude and latitude for
  * the site.
  */
  fgets(line, max, infile);  /* Skip first line of header. */
  fgets(line, max, infile);  /* The second line. */

  sscanf(line, "# %s %f %d %f %f %f",
    dummystr, &dummy, &siteId, &dummy, &hdr->lon, &hdr->lat);

  hdr->site = cnECIFSGetSite(siteId);
  if ( hdr->site < 0 )
  {
    printError(PE_EXIT, "%s: unrecognized site ID from header: %d\n",
     filename, siteId);
  }

  fgets(line, max, infile);  /* Skip third line of header. */
}


void cnECIFSGetVarLine(char * line, cnTECIFSvarLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f",
    &idate, &itime, &vdate, &vtime,
    &ln->level,
    &ln->pressure,
    &ln->uwind,
    &ln->vwind,
    &ln->temp,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->cloudFrac,
    &ln->rh,
    &ln->omega
  );

  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}


void cnECIFSGetFlxLine(char * line, cnTECIFSflxLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f",
    &idate, &itime, &vdate, &vtime,
    &ln->level,
    &ln->netSwFlx,
    &ln->netLwFlx,
    &ln->sensHeatFlx,
    &ln->turbMoistFlx,
    &ln->convRainFlx,
    &ln->convSnowFlx,
    &ln->largeRainFlx,
    &ln->largeSnowFlx,
    &ln->uTurbMomFlx,
    &ln->vTurbMomFlx
  );

  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}


void cnECIFSGetSfcLine(char * line, cnTECIFSsfcLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f"
    " %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &idate, &itime, &vdate, &vtime,
    &ln->level,
    &ln->latHeatFlx,
    &ln->sensHeatFlx,
    &ln->swRad,
    &ln->lwRad,
    &ln->clearSwRad,
    &ln->clearLwRad,
    &ln->largeRain,
    &ln->convRain,
    &ln->largeSnow,
    &ln->convSnow,
    &ln->totCloud,
    &ln->largeRainFrac,
    &ln->downSw,
    &ln->pressure,
    &ln->downLw,
    &ln->gp,
    &ln->landFrac,
    &ln->pressure2,
    &ln->temp2m,
    &ln->q2m,
    &ln->uWind10m,
    &ln->vWind10m,
    &ln->roughMom,
    &ln->roughHeat,
    &ln->albedo,
    &ln->pblHeight,
    &ln->skinTemp
  );

  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}


/************************************************************************/
/************************************************************************/
