/*
 * CloudNET Model Conversion Software
 *
 * Stuff for ECMWF model output files.
 *
 * HISTORY
 * 2004-02-24 ao started.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNECIFS_H)
#define CNECIFS_H


/************************************************************************/
/************************************************************************/


#include "cnapp.h"
#include "cnsites.h"
#include "cnutils.h"


/************************************************************************/
/************************************************************************/


/* The information from the file header. */
typedef struct
{
  int   site;  /* site ID as in 'cnsites.h'. */

  /* model lon/lat coordinates. */
  float lon;
  float lat;
} cnTECIFSheader;


/* The information from each line of data from a 'var' file. */
typedef struct
{
  int   initTime;   /* minutes since 1960. */
  int   verTime;    /* minutes since 1960. */
  int   level;
  float pressure;
  float uwind;
  float vwind;
  float temp;
  float q;
  float ql;
  float qi;
  float cloudFrac;
  float rh;
  float omega;
} cnTECIFSvarLine;


/* The information from each line of data from a 'flx' file. */
typedef struct
{
  int   initTime;   /* minutes since 1960. */
  int   verTime;    /* minutes since 1960. */
  int   level;
  float netSwFlx;
  float netLwFlx;
  float sensHeatFlx;
  float turbMoistFlx;
  float convRainFlx;
  float convSnowFlx;
  float largeRainFlx;
  float largeSnowFlx;
  float uTurbMomFlx;
  float vTurbMomFlx;
} cnTECIFSflxLine;


/* The information from each line of data from a concatenated surface
 * file.
 */
typedef struct
{
  int   initTime;   /* minutes since 1960. */
  int   verTime;    /* minutes since 1960. */
  int   level;
  float latHeatFlx;
  float sensHeatFlx;
  float swRad;
  float lwRad;
  float clearSwRad;
  float clearLwRad;
  float largeRain;
  float convRain;
  float largeSnow;
  float convSnow;
  float totCloud;
  float largeRainFrac;
  float downSw;
  float pressure;
  float downLw;
  float gp;
  float landFrac;
  float pressure2;
  float temp2m;
  float q2m;
  float uWind10m;
  float vWind10m;
  float roughMom;
  float roughHeat;
  float albedo;
  float pblHeight;
  float skinTemp;
} cnTECIFSsfcLine;


/************************************************************************/
/************************************************************************/


/* Read the first three lines from 'infile'.
 * Use 'line' as the buffer to read the lines into.
 * 'max' is the maximum length any line can be.
 */
void cnECIFSGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTECIFSheader * hdr/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnECIFSGetVarLine(char * line, cnTECIFSvarLine * ln/*OUT*/);


/* 'line' is a line read from a 'flx' file. */
void cnECIFSGetFlxLine(char * line, cnTECIFSflxLine * ln/*OUT*/);


/* 'line' is a line read from a concatenated 'sfc' file. */
void cnECIFSGetSfcLine(char * line, cnTECIFSsfcLine * ln/*OUT*/);


/************************************************************************/
/************************************************************************/


#endif
