

#include "cnknrac.h"


/************************************************************************/
/************************************************************************/


void cnKNRACGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTKNRACheader * hdr)
{
  int    date;
  int    time;
  int    i;
  char * p;
  int    model;

  /* The file header should be similar to:
01 #            KNMI   institute
02 # E van Meijgaard   participant
03 #     RACMO-ECMWF   model
04 #    CLIWA-BBC-HC   experiment
05 #  20021001  1200   reference time
06 # CA       Cabauw
07 #    4.76   51.39   rlon-rlat
08 #  61  34    3132   ilon-ilat-igrid
09 #     0.13919E+03   surface geopotential
10 #     0.99414E+00   land sea mask
11 #   vdate vhrm lev         PRES         TEMP ...
    *
    * for multi-level files and
01 #            KNMI   institute
02 # E van Meijgaard   participant
03 #     RACMO-ECMWF   model
04 #    CLIWA-BBC-HC   experiment
05 #  20021001  1200   reference time
06 # CA       Cabauw
07 #    4.76   51.39   rlon-rlat
08 #  60  33    3132   ilon-ilat-igrid
09 #     0.13919E+03   surface geopotential
10 #     0.99414E+00   land sea mask
11 #     0.18890E+00   surface albedo
12 #     0.37600E+00   roughness momentum
13 #     0.45236E-02   roughness heat/moist
14 #   vdate vhrm        PSURF         SENF ...
   *
   * for single-level files.
   */


  fgets(line, max, infile); /* first line. */

  if ( strstr(line, "SMHI") != NULL )
    {
      hdr->institute = CN_MI_SM;
      hdr->model = CN_ML_SMRCA;
      model = CN_ML_SMRCA;
    }
  else
    {
      hdr->institute = CN_MI_KN;
      hdr->model = CN_ML_KNHIR;
      model = CN_ML_KNHIR;
    }

  /* Skip next line. */
  for ( i = 0; i < 2; i++ )
  {
    fgets(line, max, infile);
  }

  /* 'line' is now the third line. */
  if ( strstr(line, "RACMO") != NULL )
    {
      hdr->model = CN_ML_KNRAC;
      model = CN_ML_KNRAC;
    }

  /* Skip next line. */
  for ( i = 0; i < 2; i++ )
  {
    fgets(line, max, infile);
  }

  /* 'line' is now the fifth line. */
  p = &line[1];
  sscanf(p, "%d %d", &date, &time);
  hdr->initTime = utToMins(date, time);

  fgets(line, max, infile); /* sixth line. */

  if ( strstr(line, "CH") != NULL )
  {
    hdr->site = CN_SITE_CH;
  }
  else
  if ( strstr(line, "CA") != NULL )
  {
    hdr->site = CN_SITE_CA;
  }
  else
  if ( strstr(line, "PA") != NULL )
  {
    hdr->site = CN_SITE_PA;
  }
  else
  {
    printError(PE_EXIT,
      "%s: unable to ascertain the site from the header\n",
      filename);
  }

  fgets(line, max, infile); /* seventh line. */
  p = &line[1];
  sscanf(p, "%f %f", &hdr->lon, &hdr->lat);

  fgets(line, max, infile); /* skip eighth line. */

  fgets(line, max, infile); /* ninth line. */
  p = &line[1];
  sscanf(p, "%f", &hdr->gp);

  fgets(line, max, infile); /* tenth line. */
  p = &line[1];
  sscanf(p, "%f", &hdr->landSeaMask);

  fgets(line, max, infile); /* eleventh line. */
  p = &line[1];
  if ( strstr(line, "vdate vhrm lev") == NULL && model == CN_ML_KNRAC )
  { /* assume single level file. */
    sscanf(p, "%f", &hdr->albedo);

    fgets(line, max, infile);
    sscanf(p, "%f", &hdr->roughMom);

    fgets(line, max, infile);
    sscanf(p, "%f", &hdr->roughHeatMoist);

    fgets(line, max, infile);
  }
}


/* 'line' is a line read from a 'var' file. */
void cnKNRACGetVarLine(char * line, cnTKNRACvarLine * ln)
{
  int date;
  int time;

  sscanf(line, "%d %d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &date, &time,
    &ln->level,
    &ln->pressure,
    &ln->temp,
    &ln->uwind,
    &ln->vwind,
    &ln->omega,
    &ln->totKinEnergy,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->cloudFrac,
    &ln->swcltau,
    &ln->lwclemis,
    &ln->prflliq,
    &ln->prflice
  );

  ln->verTime = utToMins(date, time);
}


/* 'line' is a line read from a concatenated 'sfc' file. */
void cnKNRACGetSfcLine(char * line, cnTKNRACsfcLine * ln)
{
  int date;
  int time;

  sscanf(line, "%d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &date, &time,
    &ln->pressure,
    &ln->sensHeatFlx,
    &ln->latHeatFlx,
    &ln->momFlx,
    &ln->downSwFlx,
    &ln->upSwFlx,
    &ln->downLwFlx,
    &ln->upLwFlx,
    &ln->toaDownSwFlx,
    &ln->toaUpSwFlx,
    &ln->toaUpLwFlx,
    &ln->convRain,
    &ln->largeRain,
    &ln->precipFrac,
    &ln->totCloud,
    &ln->tSkin
  );

  ln->verTime = utToMins(date, time);
}


/************************************************************************/
/************************************************************************/
