/*
 * CloudNET Model Conversion Software
 *
 * Stuff for RACMO model output files.
 *
 * HISTORY
 * 2004-02-27 ao started.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNKNRAC_H)
#define CNKNRAC_H


/************************************************************************/
/************************************************************************/


#include "cnapp.h"
#include "cnsites.h"
#include "cnmodels.h"
#include "cnutils.h"


/************************************************************************/
/************************************************************************/


/* The information from the file header. */
typedef struct
{
  int   model;     /* model ID as in 'cnmodels.h'. */
  int   institute; /* institute ID as in 'cnmodels.h'. */
  int   site;      /* site ID as in 'cnsites.h'. */

  int   initTime; /* minutes since 1960. */

  /* model lon/lat coordinates. */
  float lon;
  float lat;

  /* Available in both var and sfc files. */
  float gp;
  float landSeaMask;

  /* Available in sfc files only. */
  float albedo;
  float roughMom;
  float roughHeatMoist;
} cnTKNRACheader;


/* The information from each line of data from a 'var' file. */
typedef struct
{
  int   verTime;    /* minutes since 1960. */
  int   level;

  float pressure;
  float temp;
  float uwind;
  float vwind;
  float omega;
  float totKinEnergy;
  float q;
  float ql;
  float qi;
  float cloudFrac;
  float swcltau;
  float lwclemis;
  float prflliq;
  float prflice;
} cnTKNRACvarLine;


/* The information from each line of data from a surface file. */
typedef struct
{
  int   verTime;    /* minutes since 1960. */

  float pressure;
  float sensHeatFlx;
  float latHeatFlx;
  float momFlx;
  float downSwFlx;
  float upSwFlx;
  float downLwFlx;
  float upLwFlx;
  float toaDownSwFlx;
  float toaUpSwFlx;
  float toaUpLwFlx;
  float convRain;
  float largeRain;
  float precipFrac;
  float totCloud;
  float tSkin;
  float albedo;
  float pblheight;
} cnTKNRACsfcLine;


/************************************************************************/
/************************************************************************/


/* Read the first three lines from 'infile'.
 * Use 'line' as the buffer to read the lines into.
 * 'max' is the maximum length any line can be.
 */
void cnKNRACGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTKNRACheader * hdr/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnKNRACGetVarLine(char * line, cnTKNRACvarLine * ln/*OUT*/);


/* 'line' is a line read from a concatenated 'sfc' file. */
void cnKNRACGetSfcLine(char * line, cnTKNRACsfcLine * ln/*OUT*/);


/************************************************************************/
/************************************************************************/


#endif
