

#include "cnmfarp.h"


/************************************************************************/
/************************************************************************/


static char * findString(char * filename, char * line, char * s)
{
  char * p;
  utUpper(line);
  p = strstr(line, s);
  if ( p == NULL )
  {
    printError(PE_EXIT,
      "%s: cannot find the string \"%s\"\n", filename, s);
  }
  p += strlen(s);
  return p;
}


void cnMFARPGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTMFARPheader * hdr/*OUT*/)
{
  char * p;

  /* The file header should be similar to:
 # STATION NAME: CHILBOLTON
 # STATION COORDINATES: LONGITUDE= 358.56         LATITUDE= 51.14
 # MODEL   COORDINATES: LONGITUDE= 358.55         LATITUDE= 51.06
 # idat    itim   vdat   vtime  lev    H LEVEL      P  HIST      U  HIST..
  * Only the first and third lines are of interest.
  */
  fgets(line, max, infile);
  p = findString(filename, line, "NAME: ");
  if ( strstr(p, "CHILBOLTON") != NULL )
    {
      hdr->site = CN_SITE_CH;
    }
  else if ( strstr(p, "CABAUW") != NULL )
    {
      hdr->site = CN_SITE_CA;
    }
  else if ( strstr(p, "PALAISEAU") != NULL )
    {
      hdr->site = CN_SITE_PA;
    }
  else if ( strstr(p, "LINDENBERG") != NULL )
    {
      hdr->site = CN_SITE_LI;
    }
  else if ( strstr(p, "POTENZA") != NULL )
    {
      hdr->site = CN_SITE_PO;
    }
  else if ( strstr(p, "SGP") != NULL )
    {
      hdr->site = CN_SITE_SG;
    }
  else if ( strstr(p, "NSA") != NULL )
    {
      hdr->site = CN_SITE_NS;
    }
  else if ( strstr(p, "NAURU") != NULL )
    {
      hdr->site = CN_SITE_T1;
    }
  else if ( strstr(p, "MANUS") != NULL )
    {
      hdr->site = CN_SITE_T2;
    }
  else if ( strstr(p, "DARWIN") != NULL )
    {
      hdr->site = CN_SITE_T3;
    }
  else
  {
    printError(PE_EXIT,
      "%s: unrecognized site name in first line of header: %s\n",
      filename, p);
  }

  fgets(line, max, infile);  /* Skip the second line. */

  fgets(line, max, infile);  /* The third line. */
  p        = findString(filename, line, "LONGITUDE=");
  hdr->lon = atof(p);
  p        = findString(filename, line, "LATITUDE=");
  hdr->lat = atof(p);


  fgets(line, max, infile);  /* Fourth line of header. */
  /* Use this to determine if the sfc file is standard or has extra columns */
  hdr->sfcColumnOrder = 0;

  if ( strstr(line, "SFSSWDO") != NULL )
    {
      hdr->sfcColumnOrder = 1;
    }
}


void cnMFARPGetVarLine(char * line, cnTMFARPvarLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f %f",
    &idate, &itime, &vdate, &vtime,
    &ln->level,
    &ln->height,
    &ln->pressure,
    &ln->uwind,
    &ln->vwind,
    &ln->temp,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->cloudFrac,
    &ln->rh,
    &ln->omega
  );

  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}


void cnMFARPGetFlxLine(char * line, cnTMFARPflxLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f %f",
    &idate, &itime, &vdate, &vtime,
    &ln->level,
    &ln->height,
    &ln->netSwFlx,
    &ln->netLwFlx,
    &ln->sensHeatFlx,
    &ln->turbMoistFlx,
    &ln->convRainFlx,
    &ln->convSnowFlx,
    &ln->largeRainFlx,
    &ln->largeSnowFlx,
    &ln->uTurbMomFlx,
    &ln->uTurbMomFlx
  );

  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}


void cnMFARPGetSfcLine(char * line, cnTMFARPsfcLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f",
	 &idate, &itime, &vdate, &vtime,
	 &ln->level,
	 &ln->latHeatFlx,
	 &ln->sensHeatFlx,
	 &ln->swRad,
	 &ln->lwRad,
	 &ln->largeRain,
	 &ln->convRain,
	 &ln->largeSnow,
	 &ln->convSnow,
	 &ln->totCloud,
	 &ln->pressure
	 );
  
  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}


void cnMFARPGetSfcLinea(char * line, cnTMFARPsfcLine * ln)
{
  /* Dates from the line are in the format YYYYMMDD and times are
   * decimal times.
   */
  int   idate;
  float itime;
  int   vdate;
  float vtime;
  float dummy;

  sscanf(line, "%d %f %d %f %d %f %f %f %f %f %f %f %f %f %f %f %f",
	 &idate, &itime, &vdate, &vtime,
	 &ln->level,
	 &ln->latHeatFlx,
	 &ln->sensHeatFlx,
	 &dummy,
	 &ln->swRad,
	 &dummy,
	 &ln->lwRad,
	 &ln->largeRain,
	 &ln->convRain,
	 &ln->largeSnow,
	 &ln->convSnow,
	 &ln->totCloud,
	 &ln->pressure
	 );

  ln->initTime = utToMins(idate, utFromDecTime(itime));
  ln->verTime  = utToMins(vdate, utFromDecTime(vtime));
}

/************************************************************************/
/************************************************************************/
