/*
 * CloudNET Model Conversion Software
 *
 * Stuff for Meteo-France model output files.
 *
 * HISTORY
 * 2004-02-24  ao  started.
 *
 * 2004-06-03  ao  in structure 'cnTMFARPflxLine' changed 'heightAmsl'
 *   to 'height'.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNMFARP_H)
#define CNMFARP_H


/************************************************************************/
/************************************************************************/


#include "cnapp.h"
#include "cnsites.h"
#include "cnutils.h"


/************************************************************************/
/************************************************************************/


/* The information from the file header. */
typedef struct
{
  int   site;
  float lon;
  float lat;
  /* Some Meteo-France surface files have a different column order */
  /* Set to 0 (standard) or 1 */
  int   sfcColumnOrder; 
} cnTMFARPheader;


/* The information from each line of data from a 'var' file. */
typedef struct
{
  int   initTime;   /* seconds since 1960. */
  int   verTime;    /* seconds since 1960. */
  int   level;
  float height;
  float pressure;
  float uwind;
  float vwind;
  float temp;
  float q;
  float ql;
  float qi;
  float cloudFrac;
  float rh;
  float omega;
} cnTMFARPvarLine;


/* The information from each line of data from a 'flx' file. */
typedef struct
{
  int   initTime;   /* seconds since 1960. */
  int   verTime;    /* seconds since 1960. */
  int   level;
  float height;
  float netSwFlx;
  float netLwFlx;
  float sensHeatFlx;
  float turbMoistFlx;
  float convRainFlx;
  float convSnowFlx;
  float largeRainFlx;
  float largeSnowFlx;
  float uTurbMomFlx;
  float vTurbMomFlx;
} cnTMFARPflxLine;


/* The information from each line of data from a concatenated surface
 * file.
 */
typedef struct
{
  int   initTime;   /* seconds since 1960. */
  int   verTime;    /* seconds since 1960. */
  int   level;
  float latHeatFlx;
  float sensHeatFlx;
  float swRad;
  float lwRad;
  float largeRain;
  float convRain;
  float largeSnow;
  float convSnow;
  float totCloud;
  float pressure;
} cnTMFARPsfcLine;


/************************************************************************/
/************************************************************************/


/* Read the first three lines from 'infile'.
 * Use 'line' as the buffer to read the lines into.
 * 'max' is the maximum length any line can be.
 */
void cnMFARPGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTMFARPheader * hdr/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnMFARPGetVarLine(char * line, cnTMFARPvarLine * ln/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnMFARPGetFlxLine(char * line, cnTMFARPflxLine * ln/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnMFARPGetSfcLine(char * line, cnTMFARPsfcLine * ln/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnMFARPGetSfcLinea(char * line, cnTMFARPsfcLine * ln/*OUT*/);


/************************************************************************/
/************************************************************************/


#endif
