

#include "cnmoumm.h"


/************************************************************************/
/************************************************************************/


static char * findString(char * filename, char * line, char * s)
{
  char * p;
  utUpper(line);
  p = strstr(line, s);
  if ( p == NULL )
  {
    printError(PE_EXIT,
      "%s: cannot find the string \"%s\"\n", filename, s);
  }
  p += strlen(s);
  return p;
}


void cnMOUMMGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTMOUMMheader * hdr/*OUT*/)
{
  char * p;
  int    year;
  int    month;
  int    day;
  int    hour;
  int    min;

  /* The file header should be similar to:
%Forecast for Station   00200110 from 2002 10 02 00 00
%YYYY MM DD HH Lev   H(km)  ...
  *
  */
  fgets(line, max, infile);

  p         = findString(filename, line, "STATION ");
  hdr->site = atoi(p);
  p         = findString(filename, line, "FROM ");
  sscanf(p, "%d %d %d %d %d", &year, &month, &day, &hour, &min);


  hdr->initTime = utToMins(year * 10000 + month * 100 + day,
    hour * 100 + min);

  if ( hdr->initTime >= utToMins(20021101, 0000) )
  {
    switch ( hdr->site )
    {
      case 200101:
        hdr->site = CN_SITE_AB;
        hdr->lon  = 356.000;
        hdr->lat  = 52.420;
        break;

      case 200110:
        hdr->site = CN_SITE_CH;
        hdr->lon  = 358.520;
        hdr->lat  = 51.120;
        break;

      case 200214:
        hdr->site = CN_SITE_PA;
        hdr->lon  = 2.204;
        hdr->lat  = 48.713;
        break;

      case 200215:
        hdr->site = CN_SITE_CA;
        hdr->lon  = 4.927;
        hdr->lat  = 51.971;
        break;

      case 200216:
        hdr->site = CN_SITE_CH;
        hdr->lon  = 358.563;
        hdr->lat  = 51.145;
        break;

      default: break;
    }
  }
  else
  {
    switch ( hdr->site )
    {
      case 200101:
        hdr->site = CN_SITE_AB;
        hdr->lon  = 356.000;
        hdr->lat  = 52.420;
        break;

      case 200110:
        hdr->site = CN_SITE_CH;
        hdr->lon  = 358.520;
        hdr->lat = 51.120;
        break;

      case 200214:
        hdr->site = CN_SITE_PA;
        hdr->lon  = 2.300;
        hdr->lat  = 48.800;
	break;

      case 200215:
        hdr->site = CN_SITE_CA;
        hdr->lon  = 5.200;
        hdr->lat  = 52.200;
        break;

      case 200216:
        hdr->site = CN_SITE_CH;
        hdr->lon  = 358.600;
        hdr->lat  = 51.100;
        break;

      default: break;
    }
  }

  fgets(line, max, infile);  /* Second line. */

  /* Use this to determine if the final column is WWIND or OMEGA.
   * This is only for Profile files.
   */
  hdr->wOrOmega = ' ';

  /* search from the end for the first space. */
  p = strrchr(line, ' ');
  if ( p != NULL )
  {
    /* Increment so that it points to the first character of the last
     * column which should be either 'W' or 'O'.
     */
    p++;
    if ( *p == 'W' || *p == 'O' )
    {
      hdr->wOrOmega = *p;
    }
  }
}


void cnMOUMMGetVarLine(char * line, cnTMOUMMvarLine * ln)
{
  /* Date/time from the line are in the format YYYY MM DD HH. */
  int year;
  int month;
  int day;
  int hour;

  sscanf(line, "%d %d %d %d %d %f %f %f %f %f %f %f %f %f %f %f %f %f"
    " %f %f",
    &year, &month, &day, &hour,
    &ln->level,
    &ln->height,
    &ln->pressure,
    &ln->ff,
    &ln->ddd,
    &ln->uwind,
    &ln->vwind,
    &ln->temp,
    &ln->rhw,
    &ln->rh,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->largeCloudFrac,
    &ln->convCloudFrac,
    &ln->womega
  );

  ln->verTime = utToMins(year * 10000 + month * 100 + day,
    hour * 100);

  ln->height         *= 1000.0F;
  ln->pressure       *= 100.0F;
  ln->temp           += 273.15F;
  ln->q              /= 1000.0F;
  ln->ql             /= 1000.0F;
  ln->qi             /= 1000.0F;
  ln->rhw            /= 100.0F;
  ln->rh             /= 100.0F;
  ln->largeCloudFrac /= 100.0F;
  ln->convCloudFrac  /= 100.0F;
}


void cnMOUMMGetSfcLine(char * line, cnTMOUMMsfcLine * ln)
{
  /* Date/time from the line are in the format YYYY MM DD HH. */
  int year;
  int month;
  int day;
  int hour;

  sscanf(line, "%d %d %d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f"
    " %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &year, &month, &day, &hour,
    &ln->altitude,
    &ln->pressure,
    &ln->netSwFlx,
    &ln->toaDownSwFlx,
    &ln->toaUpSwFlx,
    &ln->downSwFlx,
    &ln->netLwFlx,
    &ln->downLwFlx,
    &ln->sfcHeatFlx,
    &ln->ustarX,
    &ln->ustarY,
    &ln->u10,
    &ln->v10,
    &ln->screenTemp,
    &ln->q1_5m,
    &ln->rh1_5m,
    &ln->td1_5m,
    &ln->mixHt,
    &ln->stableBl,
    &ln->scStable,
    &ln->wellMixed,
    &ln->decSc,
    &ln->decScCu,
    &ln->cu,
    &ln->largeRainFlx,
    &ln->largeSnowFlx,
    &ln->convRainFlx,
    &ln->convSnowFlx,
    &ln->totRain,
    &ln->convCloud,
    &ln->totCloudRnd,
    &ln->totCloudMaxRnd,
    &ln->wetFreezeHeight,
    &ln->pressureAmsl
  );

  ln->verTime = utToMins(year * 10000 + month * 100 + day,
    hour * 100);

  ln->screenTemp += 273.15F;
}


/************************************************************************/
/************************************************************************/
