/*
 * CloudNET Model Conversion Software
 *
 * Stuff for UKMO UM output files.
 *
 * HISTORY
 * 2004-02-24 ao started.
 *
 * AUTHORS
 * Andrew Openshaw
 */


#if !defined(CNMOUMM_H)
#define CNMOUMM_H


/************************************************************************/
/************************************************************************/


#include "cnapp.h"
#include "cnsites.h"
#include "cnutils.h"


/************************************************************************/
/************************************************************************/


/* The information from the file header. */
typedef struct
{
  /* Indicates whether it's wwind or omega in the last column of profile
   * files.
   * Set to 'W', 'O' or ' ' if not a Profile file.
   */
  char  wOrOmega;

  int   initTime;  /* initialization time, minutes from 1960. */

  /* Site ID */
  int   site;

  /* There's no lon/lat from the files. This will be derived from
   * tables.
   */
  float lon;
  float lat;
} cnTMOUMMheader;


/* The information from each line of data from a 'Profile' file. */
typedef struct
{
  int   verTime;          /* minutes since 1960. */
  int   level;            /* 0-based. */
  float height;           /* km in the file, converted to metres. */
  float pressure;         /* hPa in the file, converted to Pa. */
  float ff;               /* ??? units are m/s. Between 0..30. */
  float ddd;              /* ??? no units. Always int between 0..400. */
  float uwind;
  float vwind;
  float temp;             /* degC in the file, converted to K. */
  float rhw;              /* RHw(%) in the file, converted to 0..1.*/
  float rh;               /* RHi(%) in the file, converted to 0..1. */
  float q;                /* q(g/kg) in the file, converted to kg/kg. */
  float ql;               /* qcl(g/kg) in the file, converted to kg/kg. */
  float qi;               /* qcf(g/kg) in the file, converted to kg/kg. */
  float largeCloudFrac;   /* CF(%) in the file, converted to 0..1. */
  float convCloudFrac;    /* CCF(%) in the file, converted to 0..1. */
  float womega;           /* sometimes wwind, sometimes omega. */
} cnTMOUMMvarLine;


/* The information from each line of data from a 'Surface' file. */
typedef struct
{
  int   verTime;
  float altitude;            /* Orog(m) in file. */
  float pressure;            /* pstar(Pa) in file. */
  float netSwFlx;            /* Net Surf SW in file. */
  float toaDownSwFlx;
  float toaUpSwFlx;
  float downSwFlx;           /* Tot Surf SW in file. */
  float netLwFlx;            /* Net Surf LW in file. */
  float downLwFlx;           /* Tot Surf LW in file. */
  float sfcHeatFlx;
  float ustarX;
  float ustarY;
  float u10;
  float v10;
  float screenTemp;          /* T 1.5m in file. degC converted to K. */
  float q1_5m;               /* q 1.5m in file. */
  float rh1_5m;
  float td1_5m;
  float mixHt;
  float stableBl;
  float scStable;
  float wellMixed;
  float decSc;
  float decScCu;
  float cu;
  float largeRainFlx;        /* LSR in file. */
  float largeSnowFlx;        /* LSS in file. */
  float convRainFlx;         /* Cnv R in file. */
  float convSnowFlx;         /* Cnv S in file. */
  float totRain;             /* Tot R in file. */
  float convCloud;
  float totCloudRnd;
  float totCloudMaxRnd;      /* Tot C Mx/Rnd in file. */
  float wetFreezeHeight;
  float pressureAmsl;
} cnTMOUMMsfcLine;


/************************************************************************/
/************************************************************************/


/* Read the first two lines from 'infile'.
 * Use 'line' as the buffer to read the lines into.
 * 'max' is the maximum length any line can be.
 */
void cnMOUMMGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTMOUMMheader * hdr/*OUT*/);


/* 'line' is a line read from a 'Profile' file. */
void cnMOUMMGetVarLine(char * line, cnTMOUMMvarLine * ln);


/* 'line' is a line read from a 'Surface' file. */
void cnMOUMMGetSfcLine(char * line, cnTMOUMMsfcLine * ln);


/************************************************************************/
/************************************************************************/


#endif
