

#include "cnsmrca.h"


/************************************************************************/
/************************************************************************/


void cnSMRCAGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTSMRCAheader * hdr)
{
  int    date;
  int    time;
  int    i;
  char * p;

  /* The file header should be similar to:
01 #            KNMI   institute
02 # E van Meijgaard   participant
03 #     RACMO-ECMWF   model
04 #    CLIWA-BBC-HC   experiment
05 #  20021001  1200   reference time
06 # CA       Cabauw
07 #    4.76   51.39   rlon-rlat
08 #  61  34    3132   ilon-ilat-igrid
09 #     0.13919E+03   surface geopotential
10 #     0.99414E+00   land sea mask
11 #   vdate vhrm lev         PRES         TEMP ...
    *
    * for multi-level files and
01 #            KNMI   institute
02 # E van Meijgaard   participant
03 #     RACMO-ECMWF   model
04 #    CLIWA-BBC-HC   experiment
05 #  20021001  1200   reference time
06 # CA       Cabauw
07 #    4.76   51.39   rlon-rlat
08 #  60  33    3132   ilon-ilat-igrid
09 #     0.13919E+03   surface geopotential
10 #     0.99414E+00   land sea mask
11 #   vdate vhrm        PSURF         SENF ...
   *
   * for single-level files.
   */


  fgets(line, max, infile); /* first line. */

  if ( strstr(line, "SMHI") != NULL )
    {
      hdr->institute = CN_MI_SM;
      hdr->model = CN_ML_SMRCA;
    }

  /* Skip next line. */
  for ( i = 0; i < 4; i++ )
  {
    fgets(line, max, infile);
  }

  /* 'line' is now the fifth line. */
  p = &line[1];
  sscanf(p, "%d %d", &date, &time);
  hdr->initTime = utToMins(date, time);

  fgets(line, max, infile); /* sixth line. */

  if ( strstr(line, "CH") != NULL )
  {
    hdr->site = CN_SITE_CH;
  }
  else
  if ( strstr(line, "CA") != NULL )
  {
    hdr->site = CN_SITE_CA;
  }
  else
  if ( strstr(line, "LI") != NULL )
  {
    hdr->site = CN_SITE_LI;
  }
  else
  if ( strstr(line, "PA") != NULL )
  {
    hdr->site = CN_SITE_PA;
  }
  else
  if ( strstr(line, "PO") != NULL )
  {
    hdr->site = CN_SITE_PO;
  }
  else
  {
    printError(PE_EXIT,
      "%s: unable to ascertain the site from the header\n",
      filename);
  }

  fgets(line, max, infile); /* seventh line. */
  p = &line[1];
  sscanf(p, "%f %f", &hdr->lon, &hdr->lat);

  fgets(line, max, infile); /* skip eighth line. */

  fgets(line, max, infile); /* ninth line. */
  p = &line[1];
  sscanf(p, "%f", &hdr->gp);

  fgets(line, max, infile); /* tenth line. */
  p = &line[1];
  sscanf(p, "%f", &hdr->landSeaMask);

  fgets(line, max, infile); /* eleventh line. */
  if ( strstr(line, "XUR") != NULL )
    {
      hdr->filetype = 1;
    }
  else
    {
      hdr->filetype = 0;
    }
}


/* 'line' is a line read from a 'var' file. */
void cnSMRCAGetVarLine(char * line, cnTSMRCAvarLine * ln)
{
  int date;
  int time;

  sscanf(line, "%d %d %d %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &date, &time,
    &ln->level,
    &ln->pressure,
    &ln->temp,
    &ln->uwind,
    &ln->vwind,
    &ln->wwind,
    &ln->omega,
    &ln->tke,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->cloudFrac,
    &ln->cuFrac,
    &ln->height
  );

  ln->verTime = utToMins(date, time);
}


/* 'line' is a line read from a concatenated 'sfc' file. */
void cnSMRCAGetSfcLine(char * line, cnTSMRCAsfcLine * ln)
{
  int date;
  int time;

  sscanf(line, "%d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &date, &time,
    &ln->pressure,
    &ln->sensHeatFlx,
    &ln->latHeatFlx,
    &ln->momFlx,
    &ln->downSwFlx,
    &ln->upSwFlx,
    &ln->downLwFlx,
    &ln->upLwFlx,
    &ln->toaDownSwFlx,
    &ln->toaUpSwFlx,
    &ln->toaUpLwFlx,
    &ln->convRain,
    &ln->largeRain,
    &ln->tSkin,
    &ln->totCloud,
    &ln->albedo,
    &ln->pblheight
  );

  ln->verTime = utToMins(date, time);
}

/*** New files have a different layout ***/
/* 'line' is a line read from a 'var' file. */
void cnSMRCAGetVarLine_new(char * line, cnTSMRCAvarLine * ln)
{
  int date;
  int time;

  sscanf(line, "%d %d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &date, &time,
    &ln->level,
    &ln->pressure,
    &ln->height,
    &ln->temp,
    &ln->uwind,
    &ln->vwind,
    &ln->tke,
    &ln->wwind,
    &ln->q,
    &ln->ql,
    &ln->qi,
    &ln->flx_ls_rain,
    &ln->flx_ls_ice,
    &ln->cloudFrac,
    &ln->cuFrac,
    &ln->cloudFracXuR,
    &ln->cuFracXuR,
    &ln->reff,
    &ln->swtau,
    &ln->lweff,
    &ln->ERATemp
  );

  ln->verTime = utToMins(date, time);
  ln->reff = ln->reff * 1000000;
}

/* 'line' is a line read from a concatenated 'sfc' file. */
void cnSMRCAGetSfcLine_new(char * line, cnTSMRCAsfcLine * ln)
{
  int date;
  int time;

  sscanf(line, "%d %d %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
    &date, &time,
    &ln->pressure,
    &ln->sensHeatFlx,
    &ln->latHeatFlx,
    &ln->downSwFlx,
    &ln->upSwFlx,
    &ln->downLwFlx,
    &ln->upLwFlx,
    &ln->convRain,
    &ln->largeRain,
    &ln->totCloud,
    &ln->q2d,
    &ln->cw2d,
    &ln->ci2d,
    &ln->tSkin
  );

  ln->verTime = utToMins(date, time);
}

/************************************************************************/
/************************************************************************/
