/*
 * CloudNET Model Conversion Software
 *
 * Stuff for SMHI RCA model output files.
 *
 * HISTORY
 * 2004-11-03  eoc Used cnknrac.h as template
 *
 * AUTHORS
 * Andrew Openshaw
 * Ewan O'Connor
 */


#if !defined(CNSMRCA_H)
#define CNSMRCA_H


/************************************************************************/
/************************************************************************/


#include "cnapp.h"
#include "cnsites.h"
#include "cnmodels.h"
#include "cnutils.h"


/************************************************************************/
/************************************************************************/


/* The information from the file header. */
typedef struct
{
  int   model;     /* model ID as in 'cnmodels.h'. */
  int   institute; /* institute ID as in 'cnmodels.h'. */
  int   site;      /* site ID as in 'cnsites.h'. */

  int   initTime; /* minutes since 1960. */

  /* model lon/lat coordinates. */
  float lon;
  float lat;

  /* Available in both var and sfc files. */
  float gp;
  float landSeaMask;

  /* New files contain different variables */
  int   filetype;

} cnTSMRCAheader;


/* The information from each line of data from a 'var' file. */
typedef struct
{
  int   verTime;    /* minutes since 1960. */
  int   level;

  float pressure;
  float temp;
  float uwind;
  float vwind;
  float wwind;
  float omega;
  float tke;
  float q;
  float ql;
  float qi;
  float cloudFrac;
  float cuFrac;
  float height;

  float flx_ls_rain;
  float flx_ls_ice;
  float cloudFracXuR;
  float cuFracXuR;
  float reff;
  float swtau;
  float lweff;
  float ERATemp;

} cnTSMRCAvarLine;


/* The information from each line of data from a surface file. */
typedef struct
{
  int   verTime;    /* minutes since 1960. */

  float pressure;
  float sensHeatFlx;
  float latHeatFlx;
  float momFlx;
  float downSwFlx;
  float upSwFlx;
  float downLwFlx;
  float upLwFlx;
  float toaDownSwFlx;
  float toaUpSwFlx;
  float toaUpLwFlx;
  float convRain;
  float largeRain;
  float tSkin;
  float totCloud;
  float albedo;
  float pblheight;

  float q2d;
  float cw2d;
  float ci2d;

} cnTSMRCAsfcLine;


/************************************************************************/
/************************************************************************/


/* Read the first three lines from 'infile'.
 * Use 'line' as the buffer to read the lines into.
 * 'max' is the maximum length any line can be.
 */
void cnSMRCAGetHeader(char * filename, FILE * infile, char * line,
  int max, cnTSMRCAheader * hdr/*OUT*/);


/* 'line' is a line read from a 'var' file. */
void cnSMRCAGetVarLine(char * line, cnTSMRCAvarLine * ln/*OUT*/);
/* 'line' is a line read from a 'var' file. */
void cnSMRCAGetVarLine_new(char * line, cnTSMRCAvarLine * ln/*OUT*/);


/* 'line' is a line read from a concatenated 'sfc' file. */
void cnSMRCAGetSfcLine(char * line, cnTSMRCAsfcLine * ln/*OUT*/);
/* 'line' is a line read from a concatenated 'sfc' file. */
void cnSMRCAGetSfcLine_new(char * line, cnTSMRCAsfcLine * ln/*OUT*/);


/************************************************************************/
/************************************************************************/


#endif
