

#if !defined(FORTRAN_TYPES_H)
#define FORTRAN_TYPES_H

/************************************************************************/

#include <stdlib.h>

/************************************************************************/

typedef float REAL;
typedef int   INTEGER;
typedef char  LOGICAL;


/* 2-dimensional REAL array. */
typedef struct
{
  int    dim2;
  REAL * data;
} REAL2;


/* 3-dimensional REAL array. */
typedef struct
{
  int    dim23;
  int    dim3;
  REAL * data;
} REAL3;


/* 2-dimensional INTEGER array. */
typedef struct
{
  int       dim2;
  INTEGER * data;
} INTEGER2;


/* 3-dimensional INTEGER array. */
typedef struct
{
  int       dim23;
  int       dim3;
  INTEGER * data;
} INTEGER3;


/* 2-dimensional LOGICAL array. */
typedef struct
{
  int       dim2;
  LOGICAL * data;
} LOGICAL2;


/* 3-dimensional LOGICAL array. */
typedef struct
{
  int       dim23;
  int       dim3;
  LOGICAL * data;
} LOGICAL3;


/************************************************************************/


/* Frees all memory allocated for an array using the 'allocXXX()'
 * functions below.
 */
#define FREE_ARRAY(a) free(a->data); free(a)


/* Calculate the indices into a 2 and 3 dimensional array.
 * 'a' is a pointer to one of REAL2, REAL3, INTEGER2, INTEGER3,
 * LOGICAL2, LOGICAL3.
 * 'i1', 'i2', 'i3' are the indeces into each dimension of the array.
 */
#define I2(a, i1, i2)      ((i1) * (a)->dim2 + (i2))
#define I3(a, i1, i2, i3)  ((i1) * (a)->dim23 + (i2) * (a)->dim3 + (i3))


/* Access the array element at 'i1', 'i2' or
 * 'i1', 'i2', 'i3'.
 */
#define A2(a, i1, i2)      ((a)->data[I2(a, i1, i2)])
#define A3(a, i1, i2, i3)  ((a)->data[I3(a, i1, i2, i3)])


/* Returns a pointer to the array element at 'i1', 'i2' or
 * 'i1', 'i2', 'i3'.
 */
#define P2(a, i1, i2)      (&(A2(a, i1, i2)))
#define P3(a, i1, i2, i3)  (&(A3(a, i1, i2, i3)))


/* 'a' is one of REAL2, INTEGER2 or LOGICAL2.
 * 'p' is the address of the start of the array.
 * 'd1', 'd2' are the dimensions of the array.
 * After calling, the memory pointed to by 'p' can be treated as a
 * two-dimensional array.
 */
#define RESHAPE2(a, p, d1, d2) \
  (a)->data = (p); (a)->dim2 = (d2)

/* 'a' is one of REAL3, INTEGER3 or LOGICAL3.
 * 'p' is the address of the start of the array.
 * 'd1', 'd2', 'd3' are the dimensions of the array.
 * After calling, the memory pointed to by 'p' can be treated as a
 * three-dimensional array.
 */
#define RESHAPE3(a, p, d1, d2, d3) \
  (a)->data = (p); (a)->dim23 = (d2) * (d3); (a)->dim3 = (d3)


/* All these return NULL on failure. */
REAL2 *    allocReal2(int dim1, int dim2);
INTEGER2 * allocInteger2(int dim1, int dim2);
LOGICAL2 * allocLogical2(int dim1, int dim2);
REAL3 *    allocReal3(int dim1, int dim2, int dim3);
INTEGER3 * allocInteger3(int dim1, int dim2, int dim3);
LOGICAL3 * allocLogical3(int dim1, int dim2, int dim3);


/************************************************************************/

#endif
