/*
 * CloudNET Model Conversion Software
 *
 * This is the microwave propagation parameters stuff.
 * This came from Robin Hogan with some layout changes by me.
 *
 * HISTORY
 * 2004-01-05 ao added the 'is_dry' option to 'getAttenuationProfile'.
 *
 * AUTHORS
 * Andrew Openshaw
 */

#if !defined(CNMWPP_H)
#define CNMWPP_H


/************************************************************************/
/************************************************************************/


#include <math.h>
#include "cnfc.h"
#include "cnnetcdf.h"


/************************************************************************/
/************************************************************************/

double saturationVapourPressure(double temp);

double liebe_attenuation(double frequency, double temp, double pressure, double vapour_pressure);

/*
 * PRE: 'height', 'pressure', 'temp', 'specificHumidity',
 * 'attenuation_dB' and 'attenuation_dB_km' must have room for at least
 * 'numLevels' elements.
 * 'frequency' is in GHz.
 */
void getAttenuationProfile(float * height, float * pressure, float * temp,
  float * specificHumidity, int numLevels, float sigma0,
  float frequency,
  int is_saturated,
  int is_dry,
  float * attenuation_dB /*OUT*/,
  float * attenuation_dB_km /*OUT*/
);


/*
 * PRE: 'temp', 'K2' and 'liqud_atten_dB_km_g_m3' must all have at least
 * 'numLevels' elements.
 * 'frequency' is in GHz.
 */
void getLiquidProfile(float * temp, int numLevels, float frequency,
  float * K2 /*OUT*/, float * liqud_atten_dB_km_g_m3 /*OUT*/);


/************************************************************************/
/************************************************************************/


#endif
