/*
 * CloudNET Model Conversion Software
 *
 *
 * HISTORY
 *
 *
 * AUTHORS
 * Andrew Openshaw
 */


/************************************************************************/
/************************************************************************/


#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include "cnapp.h"
#include "cnfcread.h"
#include "version.h"


/************************************************************************/
/************************************************************************/


enum
{
  /* minimum number of arguments on command line after the options. */
  MINARGS = 1

};


/* Program name.
 * This is initialised in the 'main()' function.
 */
char * PROGNAME;


/* Options and arguments from the command line are here. */
struct
{



  /* Index into 'argv[]' of the first argument after the options. */
  int firstArg;
} cl;


/************************************************************************/
/************************************************************************/


/* Print the program synopsis on 'outfile'.
 * If 'exitProg'!=0 then 'exit(EXIT_FAILURE)' is called.
 */
void printSynopsis(FILE * outfile, int exitProg)
{
  fprintf(outfile,
    "\n"
    "CloudNET Model Conversion Software\n"
    "\n"
    "NAME\n"
    "\n"
    "  %s [options] file...\n"
    ,
    PROGNAME
  );
  fprintf(outfile,
    "\n"
    "DESCRIPTION\n"
    "\n"
  );
  fprintf(outfile,
    "\n"
    "OPTIONS\n"
    "\n"
    "  -h\n"
    "  -?\n"
    "    This help display.\n"
  );
  fprintf(outfile,
    "\n"
    "ARGUMENTS\n"
    "\n"
    "  'file...' are FC netCDF datasets.\n"
    "\n"
  );

  if ( exitProg )
  {
    exit(EXIT_FAILURE);
  }
}


/* Read the command line options and arguments. */
void readCommandLine(int argc, char * argv[])
{
  int argn = 1;

  if ( (argc - 1) < MINARGS )
  {
    printError(PE_PRINT_SYNOPSIS | PE_EXIT, "not enough arguments");
  }

  /* Read options. */
  while ( argn < argc && argv[argn][0] == '-' )
  {
    /* 'option' is set to the command line option after the
     * leading hyphen.
     */
    char * option = &argv[argn][1];

    /* For single character options use 'switch ( option[0] )' and a
     * 'case' statement for each option.
     * For multi-character options use 'if ( strcmp(option, "xxx") == 0)'
     * for each option.
     *
     * The 'getOptArg()' function is declared in 'cnapp.h' and is called
     * as 'getOptArg(option, argc, argv, &argn)'.
     */
    switch ( option[0] )
    {
      case 'h':
      case '?':
        printSynopsis(stdout, 1);
        break;


      default:
        printError(PE_PRINT_SYNOPSIS | PE_EXIT,
          "unrecognized option \"%s\"", option);
        break;
    }

    argn++;
  }

  /**********************************************************************/

  if ( (argc - argn) < MINARGS )
  {
    printError(PE_PRINT_SYNOPSIS | PE_EXIT, "not enough arguments");
  }

  cl.firstArg = argn;
}


/************************************************************************/
/************************************************************************/


void processFile(char * filename, char * historyLine)
{
  static cnTFCread fc;

  cnFcOpen(filename, &fc);
  cnFcAddGlobalAtt(&fc, "history", historyLine);





  NCEXIT(nc_close(fc.id));
}


/************************************************************************/
/************************************************************************/


int main(int argc, char * argv[])
{
  char * historyLine;

  /**********************************************************************/

  /* Set PROGNAME to the name of the program.
   * Remove any leading directory path.
   * Assumes UNIX directory separator.
   */
  PROGNAME = strrchr(argv[0], '/');
  PROGNAME = ( PROGNAME == NULL ) ? argv[0] : PROGNAME + 1;

  /**********************************************************************/

  /* Initialise 'cl' with default values here. */

  cl.firstArg  = -1;

  /**********************************************************************/

  readCommandLine(argc, argv);

  historyLine = ncMakeHistoryLine(NULL, argc, argv);
  for ( ; cl.firstArg < argc; cl.firstArg++ )
  {
    processFile(argv[cl.firstArg], historyLine);
  }
  free(historyLine);

  /**********************************************************************/

  return 0;
}


/************************************************************************/
/************************************************************************/

