/*
 * Calculate run a radiation calculation program to test MWPP 
 *
 * AUTHORS
 * Nicolas Gaussiat
 */




/************************************************************************/
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cnmwpp.h"

#define liebe_attenuation_RH(f, T, p, RH) \
  liebe_attenuation(f, T, p, (RH) * saturationVapourPressure(T))

void
usage(char **argv)
{
  fprintf(stderr, "Usage: %s [options] [file.dat]\n", argv[0]);
  fprintf(stderr, "Options:\n"
	  "    -verbose     Output header line and any comments in input stream\n"
	  "       -echo     Also output the input data\n"
	  "    -comment  c  Use c as comment character in input and output (default %%)\n"
	  "  -h, --help     Print this message\n");
  fprintf(stderr, "If an input file is not given on the command line, standard input is assumed\n");
  fprintf(stderr, "The parameters read in are:\n"
	  "         frequency (GHz)\n"
	  "         temperature (K)\n"
	  "         pressure (Pa)\n"
	  "         relative humidity (unitless), with respect to ice below 273.16 K\n"
	  "The parameters returned are:\n"
	  "         1-way specific gaseous attenuation (dB/km)\n");
}

int
main(int argc, char **argv)
{
  FILE *file = stdin;
  int line = 1, iarg = 0;
  char verbose = 0;
  char echo = 0;
  char comment = '%';

  while (++iarg < argc) {
    if (*argv[iarg] != '-') {
      file = fopen(argv[iarg], "r");
      if (!file) {
	fprintf(stderr, "Error reading %s\n", argv[iarg]);
	exit(1);
      }
      break;
    }
    if (strcmp(argv[iarg],"-verbose") == 0) {
      verbose = 1;
    }
    else if (strcmp(argv[iarg], "-echo") == 0) {
      echo = 1;
    }
    else if (strcmp(argv[iarg], "-comment") == 0) {
      if (argc > iarg+1) {
	comment = *argv[++iarg];
      }
    }
    else if (strcmp(argv[iarg], "-h") == 0
	     || strcmp(argv[iarg], "--help") == 0 ) {
      usage(argv);
      exit(0);
    }
    else {
      fprintf(stderr, "Error: %s not understood\n", argv[iarg]);
      usage(argv);
      exit(2);
    }
  }


  if (verbose) {
    fputc(comment, stdout);
    if (echo) {
      fprintf(stdout, " Freq(GHz) Temp(K) Pres(Pa) RH(-)");
    }
    fprintf(stdout, " Gas(dB/km)\n"); 
  }

  while (!feof(file)) {
    float frequency_GHz;
    float temperature_K;
    float pressure_Pa;
    float relative_humidity;
    float gas_attenuation_dB_km;
    int nargs;
    int ch;

    ch = fgetc(file);
    if (ch == comment) {
      if (verbose) fputc(ch, stdout);
      while ((ch = fgetc(file)) != '\n') {
	if (verbose) fputc(ch, stdout);
	if (feof(file)) {
	  if (verbose) fputc('\n', stdout);
	  break;
	}
      }
      if (verbose) fputc('\n', stdout);
    }
    else {
      ungetc(ch, file);
    }

    nargs = fscanf(file, "%g %g %g %g", &frequency_GHz, &temperature_K,
		   &pressure_Pa, &relative_humidity);
    if (nargs < 4) {
      if (feof(file)) {
	break;
      }
      else {
	fprintf(stderr, "Mangled input at line %d\n", line);
	break;
      }
    }
    if (echo) {
      fprintf(stdout, "  %9.4f %7.2f %8.1f %5.3f", frequency_GHz, temperature_K,
	      pressure_Pa, relative_humidity);
    }
    else {
      fprintf(stdout, " ");
    }
    cnFcInitialValue = -999.;
    gas_attenuation_dB_km = liebe_attenuation_RH(frequency_GHz,
						 temperature_K,
						 pressure_Pa,
						 relative_humidity);
    fprintf(stdout, " %8.4e\n", gas_attenuation_dB_km);
    line++;
  }

  exit(0);
}
